Package: scECODA
Title: Single-Cell Exploratory Compositional Data Analysis
Version: 1.1.5
Authors@R: c(
    person("Christian", "Halter",
      email = "scecoda.dev@gmail.com",
      role = c("aut", "cre"),
      comment = c(ORCID = '0009-0009-5479-2246')),
    person("Massimo", "Andreatta",
      email = "Massimo.Andreatta@unige.ch",
      role = c("aut"),
      comment = c(ORCID = '0000-0002-8036-2647')),
    person("Santiago", "Carmona",
      email = "Santiago.Carmona@unige.ch",
      role = c("aut"),
      comment = c(ORCID = '0000-0002-2495-0671')),
    person("Swiss Cancer Research Foundation", role = c("fnd"))
    )
Description: The scECODA R package provides a complete workflow for the 
    analysis and visualization of compositional data, primarily focusing on 
    cell type proportions derived from single-cell data. It implements 
    specialized methods, such as the Centered Log-Ratio (CLR) transformation, 
    to properly analyze proportional data while avoiding the biases introduced 
    by the compositional constraint. The package encapsulates data management, 
    transformation, and analysis into a single SummarizedExperiment object,
    offering downstream tools for dimensionality reduction via PCA, calculating 
    critical metrics like the Adjusted Rand Index (ARI) and Modularity to 
    quantify sample grouping quality, and generating high-quality visualizations
    like heatmaps and scatter plots.
biocViews: Software, SingleCell, Transcriptomics, CellBasedAssays,
        Normalization, Preprocessing, Visualization, Clustering,
        DimensionReduction, FeatureExtraction, PrincipalComponent
BugReports: https://github.com/carmonalab/scECODA/issues
License: GPL-3 + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Depends: R (>= 4.6.0)
VignetteBuilder: knitr
Suggests: Seurat (>= 5.0.0), igraph, knitr, rmarkdown, BiocStyle,
        testthat, scRNAseq
Config/testthat/edition: 3
URL: https://github.com/carmonalab/scECODA
Imports: BiocGenerics, cluster, corrplot, DESeq2, dplyr, factoextra (>=
        2.0.0), ggplot2, ggpubr, ggrepel, gtools, Matrix, mclust,
        methods, pheatmap, plotly, rlang, rstatix, S4Vectors, stringr,
        SummarizedExperiment (>= 1.34.0), tidyr, vegan
git_url: https://git.bioconductor.org/packages/scECODA
git_branch: devel
git_last_commit: 4af79f0
git_last_commit_date: 2026-05-13
Repository: Bioconductor 3.24
Date/Publication: 2026-05-13
NeedsCompilation: no
Packaged: 2026-05-14 00:03:09 UTC; biocbuild
Author: Christian Halter [aut, cre] (ORCID:
    <https://orcid.org/0009-0009-5479-2246>),
  Massimo Andreatta [aut] (ORCID:
    <https://orcid.org/0000-0002-8036-2647>),
  Santiago Carmona [aut] (ORCID: <https://orcid.org/0000-0002-2495-0671>),
  Swiss Cancer Research Foundation [fnd]
Maintainer: Christian Halter <scecoda.dev@gmail.com>

# Generated by roxygen2: do not edit by hand

export(calc_anosim)
export(calc_ari)
export(calc_clr)
export(calc_freq)
export(calc_modularity)
export(calc_sil)
export(calculate_pseudobulk)
export(deseq2_normalize)
export(ecoda)
export(find_hvcs)
export(get_celltype_counts)
export(get_celltype_variances)
export(get_hvcs)
export(get_sample_metadata)
export(plot_barplot)
export(plot_boxplot)
export(plot_corr)
export(plot_heatmap)
export(plot_pca)
export(plot_pca3d)
export(plot_varmean)
export(replace_zeros)
importFrom(BiocGenerics,counts)
importFrom(DESeq2,DESeqDataSetFromMatrix)
importFrom(DESeq2,estimateSizeFactors)
importFrom(DESeq2,vst)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,tcrossprod)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(cluster,pam)
importFrom(cluster,silhouette)
importFrom(corrplot,corrplot)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n_distinct)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(factoextra,fviz_pca)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_jitterdodge)
importFrom(ggplot2,scale_color_discrete)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggpubr,ggboxplot)
importFrom(ggpubr,stat_compare_means)
importFrom(ggpubr,stat_pvalue_manual)
importFrom(ggrepel,geom_text_repel)
importFrom(gtools,mixedsort)
importFrom(mclust,adjustedRandIndex)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(pheatmap,pheatmap)
importFrom(plotly,add_markers)
importFrom(plotly,add_paths)
importFrom(plotly,group2NA)
importFrom(plotly,plot_ly)
importFrom(rlang,.data)
importFrom(rlang,sym)
importFrom(rstatix,add_xy_position)
importFrom(rstatix,wilcox_test)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,hclust)
importFrom(stats,prcomp)
importFrom(stats,reformulate)
importFrom(stats,var)
importFrom(stringr,str_to_title)
importFrom(tidyr,everything)
importFrom(tidyr,pivot_longer)
importFrom(vegan,anosim)

# scECODA – single-cell Exploratory COmpositional Data Analysis

<p align="center">
<img width="154" height="154" alt="image" src="https://github.com/user-attachments/assets/ecd4f6c8-de4b-433c-b0f7-75aa2e37dee7" />
</p>

Single-cell omics technologies are increasingly applied to large patient cohorts, opening the possibility of unsupervised patient stratification based on cellular states. Identifying patient groups that share common patterns of cellular dysregulation is a key step toward more precise therapeutic strategies. 
**Cohort-level exploratory analysis of single-cell omics data** requires mapping high-dimensional molecular profiles into a lower-dimensional patient-level (or more generally, sample-level) representation space, in which biologically meaningful cohort-level structure can be identified, for example through clustering.

`scECODA` is a scalable and interpretable framework for **exploratory, cohort-level analysis and patient stratification based on single-cell transcriptomics**. It enables intuitive exploration of multi-sample datasets –such as of large patient cohorts– and facilitates the unsupervised identification of samples with similar cell type compositional profiles. In addition, `scECODA` provides **metrics to quantify the degree of separation between groups of samples** and to pinpoint the cell types or states whose change in abundance drives these differences. `scECODA` takes as input scRNA-seq count matrices (as SingleCellExperiment or Seurat objects) with cell type labels, and provides simple sample-level representations such as the **centered-log-ratio-transformed cell type compositional vectors**, as well as **pseudobulk-based representations**.


### Package Installation

To install `scECODA` from BioConductor (requires R-4.6 or higher), run the following code from within R or RStudio:

```r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
  install.packages("BiocManager")
  BiocManager::install(version = "3.23")
}
BiocManager::install("scECODA")
```

**An scECODA version compatible with R-4.4 is available here**:
``` r
install.packages("remotes")
library(remotes)
remotes::install_github("carmonalab/scECODA", ref="compatibility-R-4.4")
```

### Example
The following example uses 868 scRNA-seq samples from the blood of healthy donors (data from [Gong & Sharma *et al.*](https://pubmed.ncbi.nlm.nih.gov/39314416/)) with previously annotated cell types. It illustrates how samples naturally separate in an unsupervised manner by donor age and CMV infection status, and highlights the top cell types whose changes in abundance drive inter-sample variation.

```r
se <- ecoda(
  seurat_object, # or SingleCellExperiment object or count data
  sample_col = "sample_id", # Metadata column containing sample annotation for each cell
  celltype_col = "celltype_annotations" # Metadata column containing cell type annotations
)
plot_pca(se)
```
See also the [Tutorial](https://github.com/carmonalab/scECODA/tree/main?tab=readme-ov-file#tutorial) below.

<img width="2700" height="2100" alt="GongSharma" src="https://github.com/user-attachments/assets/aa8b34ba-722c-495d-a9f7-3aea92842652" />

Code to reproduce this figure: [https://github.com/carmonalab/scECODA/blob/main/data-raw/Create_readme_figure.rmd](https://github.com/carmonalab/scECODA/blob/main/data-raw/Create_readme_figure.rmd)


### Tutorial

Check out our step-by-step [scECODA tutorial](https://bioconductor.posit.co/packages/release/bioc/vignettes/scECODA/inst/doc/scECODA.html) ([RMD](https://github.com/carmonalab/scECODA/blob/main/vignettes/scECODA.rmd))


### Case studies

[**Case Study 1: Granularity Matters**](https://carmonalab.github.io/scECODA_demo/Case_Study_1.html) -
See how **fine-grained cell type annotation** can be crucial to uncover inter-sample biological variation missed by broad, low-resolution annotation or pseudobulk gene expression in these ECODA analyses of i) blood samples from healthy individuals and ii) lung samples from patients with different pulmonary diseases. ([RMD](https://github.com/carmonalab/scECODA_demo/blob/master/Case_Study_1.rmd))

[**Case Study 2: Cell type composition vs. Pseudo-bulk gene expression**](https://carmonalab.github.io/scECODA_demo/Case_Study_2.html) -
See how scECODA's compositional analysis compares to **pseudobulk** analysis and outperforms it when differences are driven by **low-abundance cell types** in a semi-synthetic dataset. ([RMD](https://github.com/carmonalab/scECODA_demo/blob/master/Case_Study_2.rmd))


## References

**Cell type composition drives patient stratification in single-cell RNA-seq cohorts** - Christian Halter, Massimo Andreatta, Santiago J. Carmona. bioRxiv preprint: [doi.org/10.64898/2026.03.27.714811](https://doi.org/10.64898/2026.03.27.714811) 

## ----setup, include=FALSE-----------------------------------------------------
# Solution for the "magick package is required to crop" error:
knitr::opts_chunk$set(
  echo = TRUE, # keep showing code chunks
  message = FALSE, 
  warning = FALSE,
  # This is the line that fixes the cropping issue:
  crop = NULL
)

## ----Install the scECODA package, eval=FALSE----------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("scECODA")

## ----Load scECODA-------------------------------------------------------------
library(scECODA)

## ----Load example SingleCellExperiment dataset, message=FALSE, warning=FALSE----
# Loading an example SingleCellExperiment dataset
library(scRNAseq)

all.ds <- as.data.frame(surveyDatasets())
ds <- 1
sce_object <- fetchDataset(
  all.ds[["name"]][ds],
  all.ds[["version"]][ds]
)

names(colData(sce_object))

## ----Create ECODA object------------------------------------------------------
# Create ECODA object
se <- ecoda(
  sce_object,
  sample_col = "sample",
  celltype_col = "cluster"
)

## ----Show example of CLR df---------------------------------------------------
assay(se, "clr")[1:5,1:5]

## ----Create ECODA object from Seurat object-----------------------------------
# Commented out because Seurat depends on igraph which has not been updated
# to work with R 4.6 yet.

# library(Seurat)
# 
# counts <- counts(sce_object)
# metadata <- as.data.frame(SummarizedExperiment::colData(sce_object))
# 
# seurat_object <- CreateSeuratObject(counts = counts, meta.data = metadata)
# 
# se <- ecoda(
#   seurat_object,
#   sample_col = "sample",
#   celltype_col = "cluster"
# )

## ----Load example datasets----------------------------------------------------
# Load example datasets
data(example_data)

Zhang <- example_data$Zhang
counts <- Zhang$cell_counts$authors_LR
freq <- calc_freq(counts)
metadata <- Zhang$metadata

se <- ecoda(data = counts, metadata = metadata)

se <- ecoda(data = freq, metadata = metadata)

## ----Show PCA based on cell type composition, message=FALSE, warning=FALSE----
se <- ecoda(
  data = example_data$Zhang$cell_counts$authors_LR,
  metadata = example_data$Zhang$metadata,
)

plot_pca(
  se,
  label_col = "Tissue",
  title = "PCA based on cell type composition",
  n_hv_feat_show = 5 # Shows the most highly variable features (cell types)
)

## ----PCA based on highly variable cell types----------------------------------
plot_pca(
  se,
  assay = "clr_hvc",
  label_col = "Tissue",
  title = "PCA based on highly variable cell types",
  n_hv_feat_show = nrow(metadata(se)$clr_hvc)
)

## ----3D PCA-------------------------------------------------------------------
plot_pca3d(se, label_col = "Tissue")

## ----Quantifying group separation, message=FALSE, warning=FALSE---------------
# You can get the separation scores like this:
dist_mat <- dist(assay(se, "clr"))
# Or use the pre-calculated distance matrix based on the CLR abundances:
dist_mat <- metadata(se)$sample_distances

labels <- colData(se)$Tissue

anosim_r_score <- calc_anosim(dist_mat, labels)
ari_score <- calc_ari(dist_mat, labels)
# modularity_score <- calc_modularity(dist_mat, labels)
silhouette_score <- calc_sil(dist_mat, labels)

print(paste("ANOSIM R:", anosim_r_score))
print(paste("ARI:", ari_score))
# print(paste("Modularity:", modularity_score))
print(paste("Silhouette:", silhouette_score))

## ----Box plot-----------------------------------------------------------------
plot_boxplot(se, title = "CLR Abundance by Cell Type")

## ----Grouped box plot---------------------------------------------------------
plot_boxplot(
  se,
  label_col = "Tissue",
  title = "CLR Abundance by Cell Type (with Wilcoxon Test)"
)

## ----Bar plot by group--------------------------------------------------------
# Plotting average cell type abundance by experimental group
plot_barplot(
  se,
  label_col = "Tissue",
  plot_by = "group",
  title = "Mean Relative Abundance by Condition"
)

## ----Bar plot by sample-------------------------------------------------------
# Plotting cell type abundance for each sample separately
plot_barplot(
  se,
  label_col = "Tissue",
  plot_by = "sample",
  title = "Relative Abundance for Each Sample"
)

## ----Plot heatmap example 1---------------------------------------------------
plot_heatmap(
  se,
  label_col = c("Clinical.efficacy.", "Tissue"),
  cutree_rows = 3,
  cutree_cols = 3
)

## ----Plot heatmap example 2, fig.height=16, fig.width=12----------------------
se <- ecoda(
  data = example_data$Gongsharma_full$cell_counts$authors_HR,
  metadata = example_data$Gongsharma_full$metadata
)

plot_heatmap(
  se,
  label_col = c("subject.cmv", "age_group"),
  # Additional arguments for pheatmap:
  cutree_rows = 3,
  cutree_cols = 5,
  show_colnames = FALSE
)

## ----Plot heatmap example 2 only HVCs-----------------------------------------
plot_heatmap(
  se,
  assay = "clr_hvc",
  label_col = c("subject.cmv", "age_group"),
  # Additional arguments for pheatmap:
  cutree_rows = 3,
  cutree_cols = 4,
  show_colnames = FALSE
)

## ----Correlation plot, fig.height=16, fig.width=16----------------------------
plot_corr(se)

## ----Correlation plot only HVCs, fig.height=16, fig.width=16------------------
plot_corr(se, assay = "clr_hvc")

## ----Mean-variance plot 50 percent--------------------------------------------
library(ggplot2)
plot_varmean(se, labels = "none") +
  ggtitle("Default - cell types explaining 50% variance")

## ----Mean-variance plot 80 percent--------------------------------------------
se_new <- find_hvcs(se, variance_explained = 0.8)
plot_varmean(se_new) +
  ggtitle("Cell types explaining 80% variance")

## ----Mean-variance plot top 5 HVCs--------------------------------------------
se_new <- find_hvcs(se, top_n_hvcs = 5)
plot_varmean(se_new) +
  ggtitle("Top 5 HVCs")

## ----Load dataset including pseudobulk calculation, message=FALSE, warning=FALSE----
# Loading an example SingleCellExperiment dataset
library(scRNAseq)

all.ds <- as.data.frame(surveyDatasets())
ds <- 1
sce_object <- fetchDataset(
  all.ds[["name"]][ds],
  all.ds[["version"]][ds]
)

se <- ecoda(
  data = sce_object,
  sample_col = "sample",
  celltype_col = "cluster",
  get_pb = TRUE
)

## ----PCA plot showing top 5 features------------------------------------------
plot_pca(
  se,
  label_col = "Condition",
  title = "PCA based on cell type composition",
  n_hv_feat_show = 5
)

## ----PCA plot based on pseudobulk---------------------------------------------
plot_pca(
  se,
  assay = "pb",
  label_col = "Condition",
  title = "PCA based on pseudobulk gene expression"
  #, n_hv_feat_show = 5 # optionally, show top n most highly variable genes
)

## ----Removing cell types, eval=FALSE------------------------------------------
# # Load example datasets
# data(example_data)
# Stephenson <- example_data$Stephenson
# counts <- Stephenson$cell_counts$authors_LR
# metadata <- Stephenson$metadata
# se <- ecoda(data = counts, metadata = metadata)
# 
# # Specify cell types to drop
# cts_to_drop <- c("RBC", "Platelets", "Neutrophils")
# keep <- !rownames(assay(se, "counts")) %in% cts_to_drop
# counts_subset <- assay(se, "counts")[, keep]
# colData_subset <- as.data.frame(colData(se))[keep, ]
# 
# # Re-initialize a new subset object
# # This ensures all transformations (CLR, HVCs, etc.) are recalculated correctly
# se_subset <- ecoda(data = counts_subset, metadata = colData_subset)

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

# PCA ---------------------------

#' @title Plot Principal Component Analysis and Calculate Clustering Scores
#'
#' @description Performs Principal Component Analysis (PCA) on a selected data
#'   matrix from the \code{SummarizedExperiment} object (default:
#'   CLR-transformed abundances, \code{clr}) and visualizes the results. It can
#'   also calculate and display several metrics to evaluate the separation of
#'   groups defined by \code{label_col}. It uses
#'   \link[factoextra:fviz_pca]{factoextra}.
#'
#' @details The clustering metrics (ARI, Modularity, Silhouette, ANOSIM) assess
#'   how well the sample groupings (\code{labels}) align with the underlying
#'   data structure in the feature space defined by the PCA.
#'
#' @param se A
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object.
#' @param assay Character string (default: \code{"clr"}). The name of the assay
#'   in the \code{SummarizedExperiment} object to use for PCA. Must be one of:
#'   \code{"clr"} (CLR-transformed abundances, default), \code{"clr_hvc"}
#'   (CLR-transformed abundances of only the most highly variable cell types
#'   (HVCs)), \code{"pb"} (pseudobulk gene expression), \code{"counts"} (raw
#'   counts), \code{"counts_imp"} (imputed counts), \code{"freq"} (relative
#'   frequencies), \code{"freq_imp"} (imputed frequencies), or
#'   \code{"asin_sqrt"} (arcsin-square root transformed data).
#' @param label_col Character string (optional, default: \code{NULL}). The name
#'   of a column in \code{colData(sd)} used to color and group samples in the
#'   plot, and for calculating clustering scores.
#' @param scale. Logical (default: \code{FALSE}). A value indicating whether the
#'   variables should be scaled to have unit variance before the PCA.
#' @param knn_k Integer (optional, default: \code{3}). The number of nearest
#'   neighbors (\code{k}) to use for the Shared Nearest Neighbor (SNN) graph
#'   construction, required for Modularity score calculation. If \code{NULL}, it
#'   defaults to \code{max(3, round(sqrt(N)))}, where \code{N} is the number of
#'   samples.
#' @param title Character string (optional, default: \code{NULL}). The main
#'   title for the plot. If clustering scores are calculated, they are appended
#'   to this title.
#' @param title_show_n_features Logical (optional, default: \code{TRUE}) Show
#'   the number of features (cell types or genes) used.
#' @param legend_title Character string (default: \code{"Group"}). The title for
#'   the color legend in the plot when a grouping column (\code{label_col}) is
#'   provided.
#' @param show_label_samples Logical (default: \code{FALSE}). If \code{TRUE},
#'   sample names are displayed next to the points in the plot. This
#'   automatically adds \code{"text"} to the \code{geom} parameter if it is not
#'   already present.
#' @param score_digits Integer (default: \code{3}). The number of decimal places
#'   to round the clustering and ANOSIM scores appended to the plot title.
#' @param cluster_score Logical (default: \code{TRUE}). If \code{TRUE},
#'   calculates the Adjusted Rand Index (ARI) using \code{\link{calc_ari}}.
#' @param mod_score Logical (default: \code{TRUE}). If \code{TRUE}, calculates
#'   the adjusted Modularity score using \code{\link{calc_modularity}}.
#' @param sil_score Logical (default: \code{FALSE}). If \code{TRUE}, calculates
#'   the average Silhouette width using \code{\link{calc_sil}}.
#' @param anosim_score Logical (default: \code{TRUE}). If \code{TRUE},
#'   calculates the ANOSIM statistic (R) using \code{vegan::anosim}.
#' @param anosim_permutations Integer (default: \code{99}). The number of
#'   permutations to use when calculating the ANOSIM statistic.
#' @param anosim_parallel Integer (default: \code{1}). The number of parallel
#'   processes/cores to use for the ANOSIM calculation.
#' @param ari_nclusts Integer (optional, default: \code{NULL}). The target
#'   number of clusters (\code{k}) to use for \code{hclust} and \code{pam}. If
#'   \code{NULL}, it defaults to the number of unique levels in \code{labels}.
#' @param pointsize Numeric (default: \code{3}). Size of the points in the plot.
#' @param labelsize Numeric (default: \code{4}). Size of the variable labels in
#'   the plot.
#' @param coord_equal Logical (default: \code{TRUE}). If \code{TRUE}, forces the
#'   aspect ratio of the plot to be equal.
#' @param axes Numeric vector (default: \code{c(1, 2)}). The principal
#'   components to plot (e.g., \code{c(1, 2)} for PC1 vs PC2).
#' @param invisible Character vector (default: \code{c("var", "quali")}).
#'   Elements to hide. Can include "var" (variables/cell types), "ind"
#'   (samples), or "quali" (group centroids). see the description of
#'   \link[factoextra:fviz_pca]{factoextra} for details.
#' @param geom Character string or vector (default: \code{"point"}). The
#'   geometry to be used for the plot. Allowed values are combinations of:
#'             \itemize{
#'               \item \code{"point"} to show points for individuals (samples)
#'               \item \code{"text"} to show labels for individuals (samples)
#'               \item \code{"arrow"} to show vectors for variables (features)
#'             }
#'   The default \code{"point"} plots points for both individuals and variables.
#'   Use \code{c("point", "text")} to show both points and labels for samples.
#' @param n_hv_feat_show Integer (default: \code{Inf}). Number of most highly
#'   variable features to show based on their contribution to the selected axes.
#' @param repel Logical (default: \code{TRUE}). Whether to use \code{ggrepel} to
#'   prevent label overlap for variable names.
#'
#' @return A \code{ggplot} object via \code{factoextra} visualizing the PCA
#'   results.
#'
#' @importFrom stats prcomp
#' @importFrom factoextra fviz_pca
#' @importFrom dplyr bind_rows
#' @importFrom ggplot2 ggtitle scale_shape_manual coord_equal
#'   scale_color_discrete
#' @importFrom S4Vectors metadata
#'
#' @export plot_pca
#'
#' @examples
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Zhang$cell_counts$authors_LR,
#'     metadata = example_data$Zhang$metadata,
#' )
#'
#' plot_pca(
#'     se,
#'     label_col = "Tissue",
#'     title = "PCA based on cell type composition",
#'     anosim_parallel = 1,
#'     n_hv_feat_show = 5 # Shows the most highly variable features (cell types)
#' )
#'
#' # Using only the most highly variable cell types
#' plot_pca(
#'     se,
#'     assay = "clr_hvc",
#'     label_col = "Tissue",
#'     title = "PCA based on highly variable cell types",
#'     anosim_parallel = 1,
#'     n_hv_feat_show = nrow(S4Vectors::metadata(se)$clr_hvc)
#' )
plot_pca <- function(se,
                     assay = c(
                         "clr", "counts", "counts_imp",
                         "freq", "freq_imp", "asin_sqrt",
                         "clr_hvc", "pb" # in metadata
                     ),
                     label_col = NULL,
                     scale. = FALSE,
                     title = NULL,
                     title_show_n_features = TRUE,
                     legend_title = "Group",
                     show_label_samples = FALSE,
                     score_digits = 3,
                     cluster_score = FALSE,
                     mod_score = FALSE,
                     sil_score = FALSE,
                     anosim_score = TRUE,
                     anosim_permutations = 99,
                     anosim_parallel = 1,
                     ari_nclusts = NULL,
                     knn_k = 3,
                     pointsize = 3,
                     labelsize = 4,
                     coord_equal = TRUE,
                     axes = c(1, 2),
                     invisible = c("var", "quali"),
                     geom = "point",
                     n_hv_feat_show = Inf,
                     repel = TRUE) {
    assay <- match.arg(assay)
    stopifnot(is.logical(scale.))
    stopifnot(is.logical(title_show_n_features))
    stopifnot(is.logical(show_label_samples))
    stopifnot(is.numeric(score_digits))
    stopifnot(is.logical(cluster_score))
    stopifnot(is.logical(mod_score))
    stopifnot(is.logical(sil_score))
    stopifnot(is.logical(anosim_score))
    stopifnot(is.numeric(pointsize))
    stopifnot(is.numeric(labelsize))
    stopifnot(is.logical(coord_equal))
    stopifnot(is.infinite(n_hv_feat_show) || is.numeric(n_hv_feat_show))
    stopifnot(is.logical(repel))

    feat_mat <- t(get_ecoda_assay(se, assay))

    if (any(cluster_score, mod_score, sil_score, anosim_score)) {
        dist_mat <- dist(feat_mat)
    }

    format_str <- paste0("%.", score_digits, "f")

    res.pca <- prcomp(feat_mat, scale. = scale.)


    if (!is.null(label_col)) {
        labels <- colData(se)[[label_col]]

        if (title_show_n_features) {
            if (assay == "clr_hvc") {
                title <- paste0(
                    title,
                    "\nHVCs: ", metadata(se)$top_n_hvcs,
                    " Variance explained: ",
                    round(
                        metadata(se)$variance_explained,
                        score_digits
                    )
                )
            } else if (assay == "pb") {
                title <- paste0(title, "\nNumber of genes: ", ncol(feat_mat))
            } else {
                title <- paste0(
                    title, "\nNumber of cell types: ", ncol(feat_mat)
                )
            }
        }
        if (anosim_score) {
            anosim_score <- calc_anosim(
                dist_mat, labels,
                permutations = anosim_permutations,
                parallel = anosim_parallel,
                digits = score_digits
            )
            title <- paste0(
                title, "\nANOSIM score: ", sprintf(format_str, anosim_score)
            )
        }
        if (cluster_score) {
            cluster_score <- calc_ari(
                dist_mat, labels,
                nclusts = ari_nclusts, digits = score_digits
            )
            title <- paste0(
                title, "\nARI: ", sprintf(format_str, cluster_score)
            )
        }
        if (mod_score) {
            mod_score <- calc_modularity(
                dist_mat, labels, knn_k,
                digits = score_digits
            )
            title <- paste0(
                title, "\nModularity score: ", sprintf(format_str, mod_score)
            )
        }
        if (sil_score) {
            sil_score <- calc_sil(dist_mat, labels, digits = score_digits)
            title <- paste0(
                title, "\nSilhouette score: ", sprintf(format_str, sil_score)
            )
        }
    } else {
        labels <- "none"
    }

    if (!is.infinite(n_hv_feat_show) & all(invisible %in% c("var", "quali"))) {
        invisible <- "quali"
    }

    if (show_label_samples) {
        if (!"text" %in% geom) geom <- c(geom, "text")
    }

    p <- fviz_pca(
        res.pca,
        axes = axes,
        habillage = labels,
        pointsize = pointsize,
        labelsize = labelsize,
        invisible = invisible,
        select.var = list(contrib = n_hv_feat_show),
        repel = repel,
        geom = geom
    ) +
        ggtitle(title) +
        scale_color_discrete(name = legend_title)

    if (!is.null(label_col)) {
        p <- p + scale_shape_manual(
            values = rep(19, length(unique(labels))),
            name = legend_title
        )
    }

    if (coord_equal) p <- p + coord_equal()

    return(p)
}


#' @title Plot 3-dimensional interactive Principal Component Analysis plot
#'
#' @description Performs Principal Component Analysis (PCA) on a selected data
#'   matrix from the \code{ECODA} object (default: CLR-transformed abundances,
#'   \code{clr}) and visualizes the results in 3D colored by \code{label_col}.
#'
#' @param se A
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object.
#' @param assay Character string (default: \code{"clr"}). The name of the assay
#'   in the \code{SummarizedExperiment} object to use for PCA. Must be one of:
#'   \code{"clr"} (CLR-transformed abundances, default), \code{"clr_hvc"}
#'   (CLR-transformed abundances of only the most highly variable cell types
#'   (HVCs)), \code{"pb"} (pseudobulk gene expression), \code{"counts"} (raw
#'   counts), \code{"counts_imp"} (imputed counts), \code{"freq"} (relative
#'   frequencies), \code{"freq_imp"} (imputed frequencies), or
#'   \code{"asin_sqrt"} (arcsin-square root transformed data).
#' @param label_col Character string (optional, default: \code{NULL}). The name
#'   of a column in \code{colData(se)} used to color and group samples in the
#'   plot, and for calculating clustering scores.
#' @param scale. Logical (default: \code{FALSE}). A value indicating whether the
#'   variables should be scaled to have unit variance before the PCA.
#'
#' @return An interactive 3D \code{plotly} object visualizing the PCA results.
#'
#' @importFrom stats prcomp
#' @importFrom plotly plot_ly add_markers add_paths group2NA
#'
#' @export plot_pca3d
#'
#' @examples
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Zhang$cell_counts$authors_LR,
#'     metadata = example_data$Zhang$metadata,
#' )
#'
#' plot_pca3d(se, label_col = "Tissue")
plot_pca3d <- function(se,
                       assay = c(
                           "clr", "counts", "counts_imp",
                           "freq", "freq_imp", "asin_sqrt",
                           "clr_hvc", "pb" # in metadata
                       ),
                       label_col = NULL,
                       scale. = FALSE) {
    assay <- match.arg(assay)
    stopifnot(is.logical(scale.))
    feat_mat <- t(get_ecoda_assay(se, assay))

    res.pca <- prcomp(feat_mat, scale. = scale.)

    if (!is.null(label_col)) {
        labels <- colData(se)[[label_col]]
    } else {
        labels <- "none"
    }

    df <- as.data.frame(res.pca$x)
    df$id <- seq_len(nrow(df))
    df$vs <- factor(labels)
    ms <- replicate(2, df, simplify = FALSE)
    ms[[2]]$PC3 <- min(df$PC3)
    m <- ms %>%
        bind_rows() %>%
        group2NA("id", "vs")

    p <- plot_ly(color = ~vs) %>%
        add_markers(data = df, x = ~PC1, y = ~PC2, z = ~PC3) %>%
        add_paths(data = m, x = ~PC1, y = ~PC2, z = ~PC3, opacity = 0.2)

    return(p)
}


#' Analysis of Similarities (ANOSIM) R score
#'
#' @description Calculates the ANOSIM R-statistic to test whether there is
#'   significant separation between two or more groups (defined by
#'   \code{labels}) based on the sample distance matrix (\code{dist_mat}).
#'
#' @details ANOSIM compares the mean of rank dissimilarities between groups to
#'   the mean of rank dissimilarities within groups. The R-statistic ranges from
#' -1 to 1:
#' \itemize{
#'   \item An R value close to **1** indicates clear separation of groups.
#'   \item An R value close to **0** indicates that the separation is
#'         no greater than expected by chance (i.e., poor separation).
#'   \item An R value close to **-1** indicates that within-group
#'         dissimilarities are greater than between-group dissimilarities
#'         (a very rare result).
#' }
#'
#' @param dist_mat Sample distance matrix.
#' @param labels Vector of factors or character strings. The grouping variable
#'   (the known cluster assignments) for each row in the matrix.
#' @param permutations Integer (default: \code{99}). The number of permutations
#'   to use when calculating the ANOSIM R-statistic and p-value.
#' @param parallel Integer (optional, default: \code{1}). The number
#'   of parallel processes/cores to use for the permutation testing.
#' @param digits Integer (default: \code{3}). The number of decimal places to
#'   round the R-statistic.
#'
#' @return A numeric value representing the **ANOSIM R-statistic**.
#'
#' @importFrom vegan anosim
#'
#' @export calc_anosim
#'
#' @examples
#' library(SummarizedExperiment)
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#'
#' # Extract necessary components
#' dist_mat <- dist(t(assay(se, "clr")))
#' labels <- colData(se)$subject.cmv
#'
#' # Run the calculation
#' calc_anosim(dist_mat, labels, parallel = 1)
calc_anosim <- function(dist_mat,
                        labels,
                        permutations = 99,
                        parallel = 1,
                        digits = 3) {
    stopifnot(is.numeric(permutations) && permutations >= 1)
    stopifnot(is.numeric(parallel) && parallel >= 1)
    score <- anosim(
        x = dist_mat,
        grouping = labels,
        permutations = permutations,
        parallel = parallel
    )[["statistic"]]

    return(round(score, digits))
}


#' Calculate Adjusted Rand Index (ARI)
#'
#' Calculates the Adjusted Rand Index (ARI) to measure the agreement between the
#' known cluster assignments (\code{labels}) and the cluster assignments derived
#' from two unsupervised clustering methods: Hierarchical Clustering
#' (\code{hclust}) and Partitioning Around Medoids (\code{pam}).
#'
#' @param dist_mat Sample distance matrix.
#' @param labels Vector of factors or character strings. The true or known
#'   cluster assignments for each row in the matrix.
#' @param nclusts Integer (optional, default: \code{NULL}). The target number of
#'   clusters (\code{k}) to use for \code{hclust} and \code{pam}. If
#'   \code{NULL}, it defaults to the number of unique levels in \code{labels}.
#' @param digits Integer (default: \code{3}). The number of decimal places to
#'   round the result.
#' @param return_mean Logical (default: \code{TRUE}). If \code{TRUE}, returns
#'   the mean of the ARI scores from \code{hclust} and \code{pam}. If
#'   \code{FALSE}, returns a named list with both individual scores.
#'
#' @return A numeric value (mean ARI) or a list of two ARI scores. ARI ranges
#'   from
#'         -1 (disagreement) to +1 (perfect agreement).
#'
#' @importFrom stats dist hclust cutree
#' @importFrom cluster pam
#' @importFrom mclust adjustedRandIndex
#'
#' @export calc_ari
#'
#' @examples
#' library(SummarizedExperiment)
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#'
#' # Extract necessary components
#' dist_mat <- dist(t(assay(se, "clr")))
#' labels <- colData(se)$subject.cmv
#'
#' # Run the calculation
#' calc_ari(dist_mat, labels)
calc_ari <- function(dist_mat,
                     labels,
                     nclusts = NULL,
                     digits = 3,
                     return_mean = TRUE) {
    results <- list()

    if (is.null(nclusts)) nclusts <- length(unique(labels))

    # Perform hierarchical clustering
    hc <- hclust(dist_mat, method = "ward.D2")
    clust_labels <- cutree(hc, k = nclusts)
    results[["hclust_accuracy"]] <- adjustedRandIndex(
        as.numeric(as.factor(labels)), clust_labels
    )

    # Perform PAM clustering
    clust_labels <- pam(dist_mat, k = nclusts)$cluster
    results[["pamclust_accuracy"]] <- adjustedRandIndex(
        as.numeric(as.factor(labels)), clust_labels
    )

    if (return_mean) {
        return(round(mean(unlist(results)), digits))
    } else {
        results[["hclust_accuracy"]] <- round(
            results[["hclust_accuracy"]], digits
        )
        results[["pamclust_accuracy"]] <- round(
            results[["pamclust_accuracy"]], digits
        )
        return(results)
    }
}


#' Calculate Adjusted Modularity Score
#'
#' Calculates the Modularity score for a given clustering (\code{labels}) based
#' on a Shared Nearest Neighbor (SNN) graph.The score is adjusted by the
#' theoretical maximum modularity for the number of groups to always be between
#' -0.5 (poor clustering) and +1 (excellent clustering)).
#'
#' @param dist_mat Sample distance matrix.
#' @param labels Vector of factors or character strings. The cluster assignments
#'   for each row in \code{dist_mat}.
#' @param knn_k Integer (optional, default: \code{NULL}). The number of nearest
#'   neighbors (\code{k}) used for SNN graph construction. If \code{NULL}, it
#'   defaults to \code{max(3, round(sqrt(N)))}, where \code{N} is the number of
#'   samples.
#' @param digits Integer (default: \code{3}). The number of decimal places to
#'   round the final adjusted score.
#'
#' @return A numeric value representing the adjusted modularity score,
#'   ranging from -0.5 to 1.0.
#'
#' @references
#' Brandes, Ulrik, et al.
#' "On finding graph clusterings with maximum modularity."
#' European Symposium on Algorithms. Springer, Berlin, Heidelberg, 2007.
#'
#'
#' @export calc_modularity
#'
#' @examples
#' library(SummarizedExperiment)
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#'
#' # Extract necessary components
#' dist_mat <- dist(t(assay(se, "clr")))
#' labels <- colData(se)$subject.cmv
#'
#' # Run the calculation
#' \dontrun{
#' calc_modularity(dist_mat, labels)
#' }
calc_modularity <- function(dist_mat, labels, knn_k = 3, digits = 3) {
    stopifnot(is.numeric(knn_k) && knn_k >= 2)

    if (!requireNamespace("igraph", quietly = TRUE)) {
        stop(
            "Package 'igraph' is required for this function. ",
            "Please install it."
        )
    }

    ngroups <- length(unique(labels))

    if (is.null(knn_k)) knn_k <- max(3, round(sqrt(attr(dist_mat, "Size"))))

    # Create a graph object
    knn <- compute_KNN_from_dist(dist_mat, knn_k)
    g <- compute_snn_graph(knn)

    # Compute modularity
    modularity_score <- igraph::modularity(
        g,
        membership = as.numeric(factor(labels))
    )

    # NOTE:
    # Maximum modularity depends on the number of groups:
    # max(mod) = 1 - 1 / (number of groups)
    # see Brandes, Ulrik, et al.
    # "On finding graph clusterings with maximum modularity."

    # Adjust modularity score for number of groups
    maximum_modularity_score <- 1 - (1 / ngroups)
    adjusted_modularity_score <- modularity_score / maximum_modularity_score

    return(round(adjusted_modularity_score, digits))
}


#' Compute K-Nearest Neighbors (KNN) from Distance Matrix
#'
#' Identifies the indices of the K-nearest neighbors for each sample based on a
#' pre-computed distance matrix.
#'
#' @param dist_mat A square distance matrix or an object of class \code{dist}.
#' @param knn_k Integer. The number of neighbors (\code{k}) to return for each
#'   sample.
#'
#' @return A matrix with \code{nrow(dist_mat)} rows and \code{knn_k} columns,
#'   where each row contains the indices of the nearest neighbors.
compute_KNN_from_dist <- function(dist_mat, knn_k) {
    dist_mat <- as.matrix(dist_mat)
    knn <- t(apply(dist_mat, 1, function(x) {
        order(x)[2:(knn_k + 1)]
    }))
    return(knn)
}


#' Compute Shared Nearest Neighbor (SNN) Graph
#'
#' Constructs a graph where nodes represent samples and edges are weighted
#' by the number of shared nearest neighbors (SNN) between them.
#'
#' @param knn A matrix of nearest neighbor indices, typically generated by
#'   \code{compute_KNN_from_dist}.
#'
#' @return An \code{igraph} graph object where edge weights correspond to the
#'   count of shared neighbors between nodes.
#'
#' @importFrom Matrix sparseMatrix tcrossprod
compute_snn_graph <- function(knn) {
    n <- nrow(knn)
    k <- ncol(knn)

    # 1. Create a binary sparse matrix (A)
    # Row i has a 1 at column j if j is a neighbor of i
    i_idx <- rep(seq_len(n), each = k)
    j_idx <- as.vector(t(knn))

    adj_bin <- sparseMatrix(
        i = i_idx,
        j = j_idx,
        x = 1,
        dims = c(n, n)
    )

    # 2. Compute SNN weights for ALL pairs using Matrix Multiplication
    # The dot product of row i and row j equals the count of shared '1's
    snn_matrix <- tcrossprod(adj_bin)

    # 3. Convert to igraph
    # weighted = TRUE treats the shared neighbor count as the edge weight
    g <- igraph::graph_from_adjacency_matrix(
        snn_matrix,
        mode = "undirected",
        weighted = TRUE,
        diag = FALSE
    )

    return(g)
}

#' Calculate Average Silhouette Width
#'
#' Calculates the average silhouette width for a given feature matrix, using
#' pre-defined cluster assignments (\code{labels}). This metric assesses the
#' quality of the clustering.
#'
#' @param dist_mat Sample distance matrix.
#' @param labels Vector of factors or character strings. The cluster assignments
#'   for each row in \code{dist_mat} (i.e., the grouping variable).
#' @param digits Integer (default: \code{3}). The number of decimal places to
#'   round the final score.
#'
#' @return A numeric value representing the mean silhouette width, typically
#'   ranging from -1 (poor clustering) to +1 (excellent clustering).
#'
#' @importFrom cluster silhouette
#' @importFrom stats dist
#'
#' @export calc_sil
#'
#' @examples
#' library(SummarizedExperiment)
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#'
#' # Extract necessary components
#' dist_mat <- dist(t(assay(se, "clr")))
#' labels <- colData(se)$subject.cmv
#'
#' # Run the calculation
#' calc_sil(dist_mat, labels)
calc_sil <- function(dist_mat, labels, digits = 3) {
    sils <- silhouette(
        x = as.numeric(factor(labels)),
        dist = dist_mat
    ) %>%
        as.data.frame()
    score <- mean(sils[["sil_width"]])
    return(round(score, digits))
}


# Box and bar plots ---------------------------

#' Reshapes ECODA data into a long format for plotting and analysis.
#'
#' This function takes either the relative abundance (\code{freq}) or
#' CLR-transformed abundance (\code{clr}) matrix from an
#' \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#' object, converts it from a wide (cell types x samples) to a long (sample,
#' cell type, value) format for plotting, and optionally joins it with a
#' specified column from the sample metadata.
#'
#' @param se An initialized
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object.
#' @param assay Character string (default: \code{"clr"}). The name of the assay
#'   in the \code{SummarizedExperiment} object to use for plotting. Must be one
#'   of: \code{"clr"} (CLR-transformed abundances, default), \code{"clr_hvc"}
#'   (CLR-transformed abundances of only the most highly variable cell types
#'   (HVCs)), \code{"counts"} (raw counts), \code{"counts_imp"} (imputed
#'   counts), \code{"freq"} (relative frequencies), \code{"freq_imp"} (imputed
#'   frequencies), or \code{"asin_sqrt"} (arcsin-square root transformed data).
#' @param label_col Character string (optional, default: \code{NULL}). The name
#'   of a column in \code{colData(se)} to merge into the long data frame (e.g.,
#'   "Disease_State" or "Batch"). If \code{NULL}, only the abundance data and
#'   sample/celltype IDs are returned.
#'
#' @return A tidy, long format data frame with columns:
#'         \itemize{
#'           \item \code{sample_id}: Sample identifier.
#'           \item \code{celltype}: Cell type name.
#'           \item \code{rel_abundance} or \code{clr_abundance}:
#'                 The quantitative value depending on the chosen
#'                 \code{data_slot}.
#'           \item \code{...}: Additional column specified by
#'                 \code{label_col} (if provided).
#'         }
#'
#' @importFrom methods slot slotNames
#' @importFrom dplyr %>% left_join all_of
#' @importFrom tidyr pivot_longer everything
#' @importFrom rlang sym
#' @importFrom SummarizedExperiment colData
#'
#' @seealso
#' \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
create_long_data <- function(se,
                             assay = c("clr", "freq", "asin_sqrt", "clr_hvc"),
                             label_col = NULL) {
    assay <- match.arg(assay)

    # 1. Extract data
    data_matrix <- as.data.frame(t(get_ecoda_assay(se, assay)))
    data_df <- as.data.frame(data_matrix)
    data_df <- data.frame(
        sample_id = rownames(data_df),
        data_df
    )

    # 2. Reshape the data from wide to long format
    long_data <- data_df %>%
        pivot_longer(
            cols = !all_of("sample_id"),
            names_to = "celltype",
            values_to = "value"
        )

    # 3. Prepare the metadata (only if label_col is provided)
    if (!is.null(label_col)) {
        if (!(label_col %in% colnames(colData(se)))) {
            stop("label_col '", label_col, "' not found in colData()")
        }

        metadata_df <- colData(se)
        metadata_df <- data.frame(
            sample_id = rownames(metadata_df),
            metadata_df
        )
        metadata_df <- metadata_df[, c("sample_id", label_col)]

        # 4. Join the long data with the metadata by sample_id
        plot_data <- long_data %>%
            left_join(metadata_df, by = "sample_id")
    } else {
        plot_data <- long_data
    }

    return(plot_data)
}


#' Generates a Stacked Bar Plot of Cell Type Relative Abundance.
#'
#' This function visualizes the relative abundance (composition) of cell types
#' across samples or across aggregated groups. It automatically handles data
#' preparation and ordering based on the provided parameters.
#'
#' @param se A
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object containing cell type relative frequencies in the \code{freq} assay.
#' @param label_col Character string (optional, default: \code{NULL}). The name
#'   of a column in \code{colData(se))} used to define grouping or groups
#'   (required if \code{plot_by = "group"}).
#' @param plot_by Character string (default: \code{"sample"}). Specifies whether
#'   to plot the relative abundance for each individual sample (\code{"sample"})
#'   or the average relative abundance aggregated by a group (\code{"group"})
#'   defined by \code{label_col}.
#' @param custom_sample_order Character vector (optional, default: \code{NULL}).
#'   A vector of sample IDs to enforce a specific order when \code{plot_by =
#'   "sample"}. If \code{NULL}, samples are ordered first by \code{label_col}
#'   (if provided) and then naturally.
#' @param title Character string (default: \code{""}). The main title for the
#'   plot.
#' @param facet_by_label_col Logical (default: \code{TRUE}). If \code{TRUE} and
#'   \code{plot_by = "sample"}, the plot will be faceted (split) horizontally by
#'   the categories in \code{label_col}.
#'
#' @return A \code{ggplot} object representing the stacked bar plot.
#'
#' @importFrom dplyr %>% group_by summarise mutate distinct arrange pull
#' @importFrom ggplot2 ggplot aes geom_col theme_minimal theme element_text labs
#'   facet_grid
#' @importFrom rlang sym
#' @importFrom gtools mixedsort
#' @importFrom stats reformulate
#'
#' @export plot_barplot
#'
#' @seealso \code{\link{create_long_data}},
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'
#' @examples
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Zhang$cell_counts$authors_LR,
#'     metadata = example_data$Zhang$metadata,
#' )
#'
#' plot_barplot(se)
#'
#' # Plotting average cell type abundance by experimental group
#' plot_barplot(
#'     se,
#'     label_col = "Tissue",
#'     plot_by = "group",
#'     title = "Mean Relative Abundance by Condition"
#' )
#'
#' # Plotting cell type abundance for each sample separately
#' plot_barplot(
#'     se,
#'     label_col = "Tissue",
#'     plot_by = "sample",
#'     title = "Relative Abundance for Each Sample"
#' )
plot_barplot <- function(se,
                         label_col = NULL,
                         plot_by = c("sample", "group"),
                         custom_sample_order = NULL,
                         title = "",
                         facet_by_label_col = TRUE) {
    plot_by <- match.arg(plot_by)
    stopifnot(is.logical(facet_by_label_col))

    # Use the helper function to get the long data from @freq
    plot_data <- create_long_data(
        se,
        assay = "freq",
        label_col = label_col
    )

    if (plot_by == "group") {
        if (is.null(label_col)) {
            stop("label_col must be provided when plot_by = 'group'")
        }

        # Plotting by group
        plot_df <- plot_data %>%
            group_by(.data$celltype, !!sym(label_col)) %>%
            summarise(
                mean_rel_abund = mean(.data$value, na.rm = TRUE),
                .groups = "drop"
            ) %>%
            mutate(x_var = !!sym(label_col), y_var = .data$mean_rel_abund)

        # Ensure group is a factor
        plot_df$x_var <- factor(plot_df$x_var)
    } else if (plot_by == "sample") {
        # Plotting by Sample
        plot_df <- plot_data %>%
            mutate(x_var = .data$sample_id, y_var = .data$value)

        current_levels <- unique(plot_df$x_var)

        # --- Sample Ordering Logic ---
        if (!is.null(custom_sample_order)) {
            if (!all(current_levels %in% custom_sample_order)) {
                stop(
                    "Custom sample order is incomplete",
                    "or contains unknown sample IDs"
                )
            }
            ordered_levels <- custom_sample_order
        } else {
            # DEFAULT: Sort samples by label_col (group) if provided

            if (!is.null(label_col)) {
                # 1. Get a unique list of samples
                # and their corresponding label_col
                sample_group_map <- plot_data %>%
                    distinct(.data$sample_id, !!sym(label_col))

                # 2. Order the samples by the group column first
                # (using mixedsort on its values) and then
                # by sample_id (using mixedsort on its values).
                ordered_levels <- sample_group_map %>%
                    mutate(
                        ordered_group = factor(
                            !!sym(label_col),
                            levels = mixedsort(unique(!!sym(label_col))),
                            ordered = TRUE
                        )
                    ) %>%
                    arrange(.data$ordered_group, mixedsort(.data$sample_id)) %>%
                    pull(.data$sample_id)
            } else {
                ordered_levels <- mixedsort(unique(plot_data$sample_id))
            }
        }

        # Apply the ordered factor to the x_var column
        plot_df$x_var <- factor(plot_df$x_var, levels = ordered_levels)
    }

    # --- Generate the plot ---
    p <- ggplot(
        plot_df,
        aes(x = .data$x_var, y = .data$y_var, fill = .data$celltype)
    ) +
        geom_col(position = "stack") +
        theme_minimal() +
        theme(
            axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5),
            legend.title = element_text(face = "bold")
        ) +
        labs(
            title = title,
            x = "",
            y = "Relative Abundance (%)",
            fill = "Cell Type"
        )

    if (!is.null(label_col) & plot_by == "sample" & facet_by_label_col) {
        p <- p + facet_grid(reformulate(label_col), scales = "free_x")
    }

    return(p)
}


#' Generates Boxplots for CLR-transformed Cell Type Abundances with Optional
#' Group Comparison.
#'
#' This function visualizes the distribution of CLR-transformed abundance for
#' each cell type using boxplots, optionally splitting the data by a sample
#' metadata column and performing statistical tests for comparison between
#' groups.
#'
#' \strong{Statistical Test Logic:}
#' \itemize{
#'   \item If the number of groups (\code{label_col} levels) is \strong{2}, the
#'         function uses the specified \code{stat_method}
#'         (default: "wilcox.test") and displays pairwise significance labels
#'         or stars.
#'   \item If the number of groups is \strong{3 or more}, and
#'         \code{stat_method} is "wilcox.test" or "t.test", the function
#'         automatically switches to the global non-parametric test,
#'         \strong{"kruskal.test"}, and displays the overall p-value
#'         for the comparison across all groups.
#' }
#'
#' @param se An
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object containing the CLR-transformed abundances in the \code{clr}.
#' @param assay Character string (default: \code{"clr"}). The name of the assay
#'   in the \code{SummarizedExperiment} object to use for plotting. Must be one
#'   of: \code{"clr"} (CLR-transformed abundances, default),  \code{"clr_hvc"}
#'   (CLR-transformed abundances of only the most highly variable cell types
#'   (HVCs)) or \code{"asin_sqrt"} (arcsin-square root transformed data).
#' @param label_col Character string (optional, default: \code{NULL}). The name
#'   of a column in \code{colData(se)} used to define groups for comparison. If
#'   \code{NULL}, a single boxplot is generated per cell type.
#' @param selected_celltypes Specify selected celltypes you want to plot instead
#'   of plotting boxplots for all. Note: this does not recalculate CLR-values on
#'   subset.
#' @param title Character string (default: \code{""}). The main title for the
#'   plot.
#' @param stat_method Character string (default: \code{"wilcox.test"}). The
#'   statistical method used for comparisons between 2 groups (e.g., "t.test",
#'   "wilcox.test"). Note: This is overridden by "kruskal.test" for 3+ groups.
#' @param paired Logical (default: \code{FALSE}). If \code{TRUE}, performs a
#'   paired statistical test (e.g., paired t-test or paired Wilcoxon test). Only
#'   applicable for 2-group comparisons.
#' @param signif_label Character string (default: \code{"p.signif"}). Controls
#'   how p-values are displayed for 2-group comparisons (e.g., "p.signif" for
#'   stars, "p.format" for numeric p-value).
#'
#' @return A \code{ggplot} object (enhanced by \code{ggpubr}) representing the
#'   CLR abundance boxplot, including jittered data points and dynamic
#'   significance information (pairwise stars for 2 groups, overall p-value for
#'   3+ groups).
#'
#' @importFrom ggplot2 aes geom_jitter labs theme element_text guides
#'   position_jitterdodge
#' @importFrom ggpubr ggboxplot stat_compare_means stat_pvalue_manual
#' @importFrom stringr str_to_title
#' @importFrom rlang sym
#' @importFrom rstatix wilcox_test add_xy_position
#' @importFrom stats as.formula
#'
#' @export plot_boxplot
#'
#' @seealso \code{\link{create_long_data}},
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'
#' @examples
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Zhang$cell_counts$authors_LR,
#'     metadata = example_data$Zhang$metadata,
#' )
#'
#' # 1. Boxplots for CLR abundance without grouping (no stats calculated):
#' plot_boxplot(se)
#'
#' # 2. Boxplots grouped by 'Treatment' (2 groups) and applying Wilcoxon test:
#' plot_boxplot(
#'     se,
#'     label_col = "Tissue",
#'     stat_method = "wilcox.test",
#'     title = "CLR Abundance by Tissue (with Wilcoxon Test)"
#' )
plot_boxplot <- function(se,
                         assay = c("clr", "clr_hvc", "asin_sqrt"),
                         label_col = NULL,
                         selected_celltypes = NULL,
                         title = "",
                         stat_method = "wilcox.test",
                         paired = FALSE,
                         signif_label = c("p.signif", "p.format")) {
    assay <- match.arg(assay)
    signif_label <- match.arg(signif_label)

    plot_data <- create_long_data(
        se,
        assay = assay,
        label_col = label_col
    )

    if (!is.null(selected_celltypes)) {
        if (!all(selected_celltypes %in% unique(plot_data$celltype))) {
            stop(
                "Not all selected_celltypes found in assay(se", assay, "). "
            )
        }

        plot_data <- plot_data[plot_data$celltype %in% selected_celltypes, ]
    }

    # Ensure celltype is a factor for plotting
    plot_data$celltype <- factor(plot_data$celltype)

    # --- Setup for Plotting ---

    # Determine color mapping for ggboxplot
    box_color_map <- if (!is.null(label_col)) label_col else "black"

    # Generate the base boxplot
    p <- ggboxplot(plot_data,
        x = "celltype",
        y = "value",
        xlab = "",
        ylab = "Abundance",
        outlier.shape = NA,
        color = box_color_map # Color by group variable or "black"
    )

    # --- Add Jittered Points and Significance ---

    if (is.null(label_col)) {
        # SCENARIO 1: No grouping variable provided
        # (Single boxplot per cell type)

        # Add simple jitter (no dodging)
        p <- p + geom_jitter(
            width = 0.2,
            size = 1,
            alpha = 0.6
        ) +
            guides(color = "none")
    } else {
        # SCENARIO 2: Grouping variable is provided (Comparison between groups)

        # Calculate the number of boxplots for correct jitter-dodging
        nr_of_boxplots <- length(unique(plot_data[[label_col]]))
        label_col_sym <- sym(label_col)

        # Add jittered points with dodging
        p <- p + geom_jitter(
            mapping = aes(color = !!label_col_sym), # Map color to group
            position = position_jitterdodge(jitter.width = 1 / nr_of_boxplots),
            size = 1,
            alpha = 0.6
        )

        # Add significance testing
        if (stat_method == "kruskal.test") {
            # Kruskal-Wallis: Overall test (no 'group' aesthetic needed)
            p <- p + stat_compare_means(
                method = stat_method,
                label.y.npc = "top", # Place label at the top
                label.x.npc = "center",
                label = "p.format" # Show the overall p-value
            )
        } else if (nr_of_boxplots == 2) {
            # Wilcoxon or t.test: Pairwise test (requires 'group' aesthetic)
            p <- p + stat_compare_means(
                aes(group = !!label_col_sym),
                method = stat_method,
                paired = paired,
                label = signif_label, # Show significance stars
                tip.length = 0,
                hide.ns = TRUE
            )
        } else if (nr_of_boxplots > 2) {
            y_var <- colnames(plot_data)[3] # "value"
            x_group_var <- colnames(plot_data)[2] # "celltype"
            fill_compare_var <- colnames(plot_data)[4] # label_col

            dsub_stats <- plot_data %>%
                group_by(!!sym(x_group_var)) %>%
                wilcox_test(as.formula(paste(y_var, "~", fill_compare_var))) %>%
                add_xy_position(x = .data$x_group_var)

            p <- p +
                # Add p-values using the generated stats table
                stat_pvalue_manual(dsub_stats,
                    label = "p.adj.signif",
                    tip.length = 0.01
                )
        }

        # Add legend title
        p <- p + labs(color = str_to_title(label_col))
    }

    # --- Final Theme and Labels ---
    p <- p +
        theme(
            axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5, ),
            legend.title = element_text(face = "bold")
        ) +
        labs(title = title)

    return(p)
}


# Heatmap ---------------------------

#' Generates a Heatmap of Cell Abundance Data from an SummarizedExperiment
#' assay.
#'
#' This function visualizes a data matrix from a specified slot (e.g.,
#' CLR-transformed, frequency, or pseudobulk data) after mean-centering. It
#' supports optional filtering to only Highly Variable Cell Types (HVCs) and
#' includes a sample annotation sidebar based on a specified metadata column.
#'
#' @param se A
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object.
#' @param assay Character string (default: \code{"clr"}). The name of the assay
#'   in the \code{SummarizedExperiment} object to use for the heatmap. Must be
#'   one of: \code{"clr"} (CLR-transformed abundances, default),
#'   \code{"clr_hvc"} (CLR-transformed abundances of only the most highly
#'   variable cell types (HVCs)), \code{"pb"} (pseudobulk gene expression),
#'   \code{"counts"} (raw counts), \code{"counts_imp"} (imputed counts),
#'   \code{"freq"} (relative frequencies), \code{"freq_imp"} (imputed
#'   frequencies), or \code{"asin_sqrt"} (arcsin-square root transformed data).
#' @param label_col Character string. The name of the column in
#'   \code{colData(se)} to use for annotating the samples (columns) of the
#'   heatmap.
#' @param cluster_rows Logical (default: \code{TRUE}). Whether to apply
#'   hierarchical clustering to the cell types (rows).
#' @param cluster_cols Logical (default: \code{TRUE}). Whether to apply
#'   hierarchical clustering to the samples (columns).
#' @param scale Character string (default: \code{"none"}). Method for scaling
#'   the abundance values within the heatmap using \code{pheatmap}. Options
#'   include \code{"none"}, \code{"row"}, or \code{"column"}. Note: The data is
#'   internally **mean-centered** (\code{scale(center=TRUE, scale=FALSE)})
#'   across samples before being passed to \code{pheatmap}, regardless of this
#'   argument.
#' @param clustering_method Character string (default: \code{"ward.D2"}). The
#'   clustering method to use for hierarchical clustering. Options are passed
#'   directly to \code{hclust} (e.g., \code{"complete"}, \code{"average"},
#'   \code{"ward.D2"}).
#' @param angle_col Character string (default: \code{"90"}). Angle of the sample
#'   labels (columns).
#' @param ... Additional arguments passed directly to the \code{pheatmap}
#'   function.
#'
#' @return A \code{pheatmap} object, which is typically visualized automatically
#'   when called interactively.
#'
#' @importFrom dplyr %>%
#' @importFrom pheatmap pheatmap
#'
#' @export plot_heatmap
#'
#' @seealso
#' \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment},
#' \code{\link[pheatmap]{pheatmap}}
#'
#' @examples
#' # Example for a simple dataset:
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Zhang$cell_counts$authors_LR,
#'     metadata = example_data$Zhang$metadata,
#' )
#'
#' plot_heatmap(se, label_col = c("Clinical.efficacy.", "Tissue"))
#'
#' plot_heatmap(
#'     se,
#'     label_col = c("Clinical.efficacy.", "Tissue"),
#'     # Additional arguments for pheatmap:
#'     cutree_rows = 3,
#'     cutree_cols = 3
#' )
#'
#' # Example of a large cohort with 868 samples and 69 cell types
#' se <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#'
#' plot_heatmap(
#'     se,
#'     label_col = c("subject.cmv", "age_group"),
#'     cutree_rows = 3,
#'     cutree_cols = 5,
#'     show_colnames = FALSE
#' )
#'
#' # Using only the most highly variable cell types (HVCs)
#' plot_heatmap(
#'     se,
#'     assay = "clr_hvc",
#'     label_col = c("subject.cmv", "age_group"),
#'     cutree_rows = 3,
#'     cutree_cols = 4,
#'     show_colnames = FALSE
#' )
plot_heatmap <- function(se,
                         assay = c(
                             "clr", "counts", "counts_imp",
                             "freq", "freq_imp", "asin_sqrt",
                             "clr_hvc", "pb" # in metadata
                         ),
                         label_col,
                         cluster_rows = TRUE,
                         cluster_cols = TRUE,
                         scale = "none",
                         clustering_method = "ward.D2",
                         angle_col = "90",
                         ...) {
    assay <- match.arg(assay)
    stopifnot(is.logical(cluster_rows))
    stopifnot(is.logical(cluster_cols))
    df_heatmap <- t(get_ecoda_assay(se, assay))

    df_heatmap <- df_heatmap %>%
        scale(center = TRUE, scale = FALSE) %>%
        t() %>%
        as.data.frame()

    metadata <- as.data.frame(colData(se)[, label_col, drop = FALSE])
    metadata[] <- lapply(metadata, as.factor)

    heatmap <- pheatmap(
        df_heatmap,
        annotation_col = metadata,
        cluster_rows = cluster_rows,
        cluster_cols = cluster_cols,
        scale = scale,
        clustering_method = clustering_method,
        angle_col = angle_col,
        ...
    )

    return(heatmap)
}


# Correlation plot ---------------------------

#' @title Plot Cell Type Correlation Matrix
#'
#' @description Calculates the pairwise Pearson correlation matrix for all cell
#'   types (columns) using the Centered Log-Ratio (CLR) transformed abundances
#'   stored in \code{assay(se, "clr")}. It then visualizes this matrix as a
#'   heatmap using \code{corrplot::corrplot}.
#'
#' @details The function uses the CLR matrix, where high correlation between two
#'   cell types suggests they vary together across samples, indicating potential
#'   co-occurrence or co-regulation.
#'
#' @param se A
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object.
#' @param assay Character string (default: \code{"clr"}). The name of the assay
#'   in the \code{SummarizedExperiment} object to use for the correlation plot
#'   Must be one of: \code{"clr"} (CLR-transformed abundances, default),
#'   \code{"clr_hvc"} (CLR-transformed abundances of only the most highly
#'   variable cell types (HVCs)), \code{"pb"} (pseudobulk gene expression),
#'   \code{"counts"} (raw counts), \code{"counts_imp"} (imputed counts),
#'   \code{"freq"} (relative frequencies), \code{"freq_imp"} (imputed
#'   frequencies), or \code{"asin_sqrt"} (arcsin-square root transformed data).
#' @param order Character string (default: \code{"hclust"}). The ordering method
#'   for the correlation matrix. Common options include:
#'              \itemize{
#'                \item \code{"original"} (no reordering)
#'                \item \code{"hclust"} (hierarchical clustering)
#'                \item \code{"FPC"} (first principal component order)
#'              }
#' @param hclust.method Character string (default: \code{"ward.D2"}). The
#'   hierarchical clustering method to use if \code{order} is set to
#'   \code{"hclust"}.
#' @param ... Additional arguments passed to \code{corrplot::corrplot} for plot
#'   customization (e.g., \code{method}, \code{type}, \code{tl.col}).
#'
#' @return A plot object generated by \code{corrplot::corrplot}, which is a base
#'   R plot or a grid object depending on the `corrplot` version and options.
#'
#' @importFrom stats cor
#' @importFrom corrplot corrplot
#'
#' @export plot_corr
#'
#' @examples
#' data(example_data)
#' # Example of a large cohort with 868 samples and 69 cell types
#' se <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#' plot_corr(se)
plot_corr <- function(se,
                      assay = c(
                          "clr", "counts", "counts_imp",
                          "freq", "freq_imp", "asin_sqrt",
                          "clr_hvc", "pb" # in metadata
                      ),
                      order = "hclust",
                      hclust.method = "ward.D2",
                      ...) {
    assay <- match.arg(assay)
    feat_mat <- t(get_ecoda_assay(se, assay))

    cor_matrix <- cor(feat_mat)

    corrplot(cor_matrix, order = "hclust", hclust.method = "ward.D2", ...)
}


# Helper function ---------------------------

#' @title Helper to get assay for ecoda SummarizedExperiment objects
#'
#' @description Because not all data can be stored in assays (e.g. pseudobulk
#'   and clr_hvc have different number of features (rows) than counts etc.)
#'
#' @param se A
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object.
#' @param assay Character string, the name of the assay.
#'
#' @return A data frame containing the assay data.
#'
#' @importFrom SummarizedExperiment assays
#' @importFrom S4Vectors metadata
get_ecoda_assay <- function(se, assay) {
    if (assay %in% names(assays(se))) {
        df <- assay(se, assay)
    } else if (assay %in% names(metadata(se))) {
        df <- metadata(se)[[assay]]
    } else {
        stop("Assay '", assay, "' not found in assays or metadata.")
    }

    return(df)
}

test_that("Clustering and distance metrics return valid scores", {
    # Create two distinct clusters in 2D space
    # All scores should be high and near 1
    set.seed(42)
    mat <- rbind(
        matrix(rnorm(10, mean = 0), ncol = 2),
        matrix(rnorm(10, mean = 100), ncol = 2)
    )
    mat <- dist(mat)
    labels <- c(rep("Group1", 5), rep("Group2", 5))

    # ANOSIM
    ano <- calc_anosim(mat, labels, parallel = 1)
    expect_true(ano > 0.8)

    # ARI
    ari <- calc_ari(mat, labels)
    expect_type(ari, "double")
    expect_true(ari > 0.8)

    # # Modularity
    # mod <- calc_modularity(mat, labels)
    # expect_true(mod > 0.8)

    # Silhouette
    sil <- calc_sil(mat, labels)
    expect_true(sil > 0.8)


    # Create two mixed clusters in 2D space
    # All scores should be low and near 0
    mat <- rbind(
        matrix(rnorm(10, mean = 0), ncol = 2),
        matrix(rnorm(10, mean = 0), ncol = 2)
    )
    mat <- dist(mat)

    # ANOSIM
    ano <- calc_anosim(mat, labels, parallel = 1)
    expect_true(ano < 0.2)

    # ARI
    ari <- calc_ari(mat, labels)
    expect_type(ari, "double")
    expect_true(ari < 0.2)

    # # Modularity
    # mod <- calc_modularity(mat, labels)
    # expect_true(mod < 0.2)

    # Silhouette
    sil <- calc_sil(mat, labels)
    expect_true(sil < 0.2)
})

test_that("create_long_data reshapes ECODA assays correctly", {
    data(example_data)
    d <- example_data$Zhang
    se <- ecoda(data = d$cell_counts$authors_LR, metadata = d$metadata)

    # Test with label join
    long_df <- scECODA:::create_long_data(se, assay = "clr", label_col = "Tissue")

    expect_true("value" %in% colnames(long_df))
    expect_true("Tissue" %in% colnames(long_df))
    expect_equal(nrow(long_df), nrow(assay(se, "clr")) * ncol(assay(se, "clr")))
})

test_that("Plotting functions return ggplot or pheatmap objects", {
    data(example_data)
    d <- example_data$Zhang
    se <- ecoda(data = d$cell_counts$authors_LR, metadata = d$metadata)

    p_pca <- plot_pca(se, label_col = "Tissue", anosim_parallel = 1)
    expect_s3_class(p_pca, "ggplot")

    p_box <- plot_boxplot(se, label_col = "Tissue")
    expect_s3_class(p_box, "ggplot")
})

# Create SummarizedExperiment object ---------------------------

# Define SummarizedExperiment object for ECODA

#' Create an SummarizedExperiment object from various data types
#'
#' This is a smart constructor function used to initialize an
#' \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#' object. It handles the processing of single-cell objects (\code{Seurat} or
#' \code{SingleCellExperiment}) or raw data frames to extract cell type counts,
#' calculate sample metadata, and optionally generate DESeq2-normalized
#' pseudobulk data.
#'
#' @param data The primary input, which can be:
#'   \itemize{
#'     \item A \code{Seurat} object.
#'     \item A \code{SingleCellExperiment} object.
#'     \item A cell type x sample matrix or data frame (counts or frequencies).
#'   }
#' @param data_is_freq Logical (default: \code{FALSE}). If \code{TRUE}, the
#'   input \code{data} is treated as relative frequencies (proportions) rather
#'   than raw counts.
#' @param metadata A data frame containing sample-level metadata. Required if
#'   \code{data} is a matrix/data frame. If \code{data} is a single-cell object,
#'   metadata is extracted automatically.
#' @param sample_col The metadata column name defining unique sample IDs.
#'   Required if \code{data} is a single-cell object.
#' @param celltype_col The metadata column name defining cell type annotations.
#'   Required if \code{data} is a single-cell object.
#' @param get_pb Logical (default: \code{FALSE}). If \code{TRUE}, calculates and
#'   stores DESeq2-normalized pseudobulk data in \code{metadata(se)$pb}
#' @param variance_explained Numeric (default: 0.5). Proportion of variance used
#'   to determine the number of highly variable cell types (HVCs).
#' @param top_n_hvcs Integer (optional). If provided, specifies the exact number
#'   of top HVCs to select, overriding \code{variance_explained}.
#' @param pseudo_count Numeric (default: 0.5). Used if \code{rep_method =
#'   "counts"}.
#' @param pseudo_frac_min Numeric (default: 2/3). The multiplier for the minimum
#'   non-zero value when \code{rep_method = "frac_min"}.
#' @param add_to Character string. Should the value be added to \code{"all"}
#'   cells or just the \code{"zeros"}?
#'
#' @return A new
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object.
#'
#' @importFrom methods new
#' @importFrom gtools mixedsort
#' @importFrom SummarizedExperiment colData colData<- assay
#' @importFrom S4Vectors metadata metadata<- DataFrame
#'
#' @export ecoda
#'
#' @examples
#' data(example_data)
#' Zhang <- example_data$Zhang
#' counts <- Zhang$cell_counts$authors_LR
#' freq <- calc_freq(counts)
#' metadata <- Zhang$metadata
#'
#' se <- ecoda(data = counts, metadata = metadata)
#'
#' se <- ecoda(data = freq, metadata = metadata)
ecoda <- function(data = NULL,
                  data_is_freq = FALSE,
                  metadata = NULL,
                  sample_col = NULL,
                  celltype_col = NULL,
                  get_pb = FALSE,
                  variance_explained = 0.5,
                  top_n_hvcs = NULL,
                  pseudo_count = 0.5,
                  pseudo_frac_min = 2 / 3,
                  add_to = NULL) {
    if (is.null(data)) stop("'data' must be provided.")
    stopifnot(is.logical(data_is_freq))
    stopifnot(is.logical(get_pb))


    # --- A) Handle input from a Seurat or SingleCellExperiment object ---
    if (inherits(data, "Seurat") || inherits(data, "SingleCellExperiment")) {
        if (is.null(sample_col) || is.null(celltype_col)) {
            stop("Please provide 'sample_col' and 'celltype_col'.")
        }

        # Get cell metadata from the object
        if (inherits(data, "Seurat")) {
            cell_data_df <- as.data.frame(slot(data, "meta.data"))
            if (get_pb) {
                if (!is.null(data[["RNA"]]$counts)) {
                    pb <- calculate_pseudobulk(
                        count_matrix = data[["RNA"]]$counts,
                        sample_ids = cell_data_df[[sample_col]]
                    )
                } else {
                    warning("Could not find RNA counts in Seurat object")
                }
            }
        } else if (inherits(data, "SingleCellExperiment")) {
            if (!requireNamespace("SummarizedExperiment", quietly = TRUE)) {
                stop("Package 'SummarizedExperiment' must be installed.")
            }
            cell_data_df <- as.data.frame(colData(data))
            names(cell_data_df) <- names(colData(data))
            if (get_pb) {
                if (!is.null(assay(data, "counts"))) {
                    pb <- calculate_pseudobulk(
                        count_matrix = assay(
                            data, "counts"
                        ),
                        sample_ids = cell_data_df[[sample_col]]
                    )
                } else {
                    warning(
                        "Could not find RNA counts ",
                        "in SingleCellExperiment object"
                    )
                }
            }
        }

        data <- get_celltype_counts(cell_data_df, sample_col, celltype_col)
        metadata <- get_sample_metadata(cell_data_df, sample_col)
    } else {
        # --- 3. Handle Matrix/DF inputs ---
        if (!is.matrix(data) && !is.data.frame(data)) {
            stop("'data' must be a Seurat/SCE object, matrix, or data frame.")
        }
    }

    data <- data[, mixedsort(colnames(data))]

    if (!is.null(metadata)) {
        metadata <- metadata[mixedsort(rownames(metadata)), , drop = FALSE]

        # Ensure correct and matching colnames
        if (!all(colnames(data) == rownames(metadata))) {
            stop(
                "Colnames of cell counts (or freq) ",
                "do not match rownames of metadata."
            )
        }
    }

    se <- ecoda_helper(
        data = data,
        data_is_freq = data_is_freq,
        variance_explained = variance_explained,
        top_n_hvcs = top_n_hvcs,
        pseudo_count = pseudo_count,
        pseudo_frac_min = pseudo_frac_min,
        add_to = add_to
    )
    if (!is.null(metadata)) colData(se) <- DataFrame(metadata)

    if (get_pb && exists("pb")) {
        pb <- deseq2_normalize(pb)
        pb <- pb[, mixedsort(colnames(pb))]
        metadata(se)$pb <- pb
    }

    return(se)
}


#' Core constructor for SummarizedExperiment objects from count/frequency
#' matrices
#'
#' This is the internal engine that initializes an
#' \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#' object. It performs zero-imputation, Centered Log-Ratio (CLR) transformation,
#' calculates sample distances, and identifies highly variable cell types
#' (HVCs).
#'
#' @param data A matrix or data frame where **columns are samples** and
#'   **rows are cell types**.
#' @param data_is_freq Logical. If \code{TRUE}, \code{data} is treated as
#'   proportions (0-1 or 0-100). If \code{FALSE}, treated as raw integer counts.
#' @param variance_explained Numeric (default: 0.5). Target variance for HVCs.
#' @param top_n_hvcs Integer (optional). Number of top HVCs to select.
#' @param pseudo_count Numeric (default: 0.5). Used if \code{rep_method =
#'   "counts"}.
#' @param pseudo_frac_min Numeric (default: 2/3). The multiplier for the minimum
#'   non-zero value when \code{rep_method = "frac_min"}.
#' @param add_to Character string. Should the value be added to \code{"all"}
#'   cells or just the \code{"zeros"}?
#'
#' @return A fully initialized
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object.
#'
#' @importFrom methods new
#' @importFrom dplyr %>% mutate across everything
#' @importFrom stats dist
#' @importFrom SummarizedExperiment SummarizedExperiment assay assay<-
#' @importFrom S4Vectors metadata metadata<-
ecoda_helper <- function(data = NULL,
                         data_is_freq,
                         variance_explained = 0.5,
                         top_n_hvcs = NULL,
                         pseudo_count = 0.5,
                         pseudo_frac_min = 2 / 3,
                         add_to = c("all", "zeros")) {
    rep_method <- ifelse(data_is_freq, "frac_min", "counts")
    add_to <- match.arg(add_to)

    data_imp <- data
    if (any(data == 0)) {
        if (data_is_freq) {
            message(
                "Frequencies contain zeros. Zeros were replaced with the (",
                pseudo_frac_min, ") * smallest value."
            )
        } else {
            message(
                "Counts contain zeros. A pseudo count of +", pseudo_count,
                " was added to ", add_to, " counts."
            )
        }

        data_imp <- replace_zeros(
            df = data,
            rep_method = rep_method,
            pseudo_count = pseudo_count,
            pseudo_frac_min = pseudo_frac_min,
            add_to = add_to
        )
    }

    if (data_is_freq) {
        freq <- data
        freq_imp <- data_imp
    } else {
        freq <- calc_freq(data)
        freq_imp <- calc_freq(data_imp)
    }

    # Check if all column sums are close to 100
    # (using a tolerance for floating point numbers)
    # If they are intended to be percentages
    colw_sums <- colSums(freq)
    if (!all(abs(colw_sums - 100) < 1e-6)) {
        # Check if they are close to 1 and should be scaled to 100
        if (all(abs(colw_sums - 1) < 1e-6)) {
            message(
                "Frequencies sum close to 1. Rescaling all rows to sum to 100."
            )
            # Scale by 100
            freq <- freq * 100
            freq_imp <- freq_imp * 100
        } else {
            # If they sum to neither 1 nor 100, warn and scale them to 100
            message(
                "Frequencies do not sum to 100 (or 1). ",
                "Each row will be scaled so the new row sum is 100."
            )
            # Re-scale each row so it sums to 100: (freq / row_sum) * 100
            freq <- (freq / colw_sums) * 100
            freq_imp <- (freq_imp / colSums(freq_imp)) * 100
        }
    }

    if (data_is_freq) {
        se <- SummarizedExperiment(
            assays = list(
                freq = freq,
                freq_imp = freq_imp
            )
        )
    } else {
        se <- SummarizedExperiment(
            assays = list(
                counts = data,
                counts_imp = data_imp,
                freq = freq,
                freq_imp = freq_imp
            )
        )
    }

    clr_df <- calc_clr(freq_imp)
    assay(se, "clr") <- clr_df
    metadata(se)$sample_distances <- clr_df %>%
        t() %>%
        dist()

    assay(se, "asin_sqrt") <- asin(sqrt(as.matrix(freq) / 100))

    se <- find_hvcs(
        se,
        variance_explained,
        top_n_hvcs
    )

    metadata(se)$clr_hvc <-
        calc_clr(freq_imp[metadata(se)$hvcs, , drop = FALSE])

    return(se)
}


#' Calculate relative frequencies (percentages) column-wise.
#'
#' This function takes a matrix or data frame of counts and transforms each
#' column (assumed to be a sample or observation) into relative frequencies,
#' expressing each component as a percentage of the column's total sum.
#'
#' @param df A data frame or matrix of counts (samples/observations as columns,
#'   components as rows). Must contain only numeric, non-negative values.
#' @return A data frame of the same dimensions, where each column sums to 100.
#' @export calc_freq
#' @examples
#' counts <- data.frame(A = c(10, 90), B = c(50, 50))
#' calc_freq(counts)
calc_freq <- function(df) {
    df <- sweep(df, 2, colSums(df), "/") * 100
    return(as.data.frame(df))
}


#' Perform the Centered Log-Ratio (CLR) transformation.
#'
#' The CLR transformation is a common technique used for compositional data
#' analysis, mapping the data from the simplex to Euclidean space. It is defined
#' as the log of the ratio between each component and the geometric mean of all
#' components in that sample. **Note:** This function assumes the input data is
#' strictly positive (i.e., does not contain zeros). Use an imputation or
#' pseudocount method prior to this function if zeros are present.
#'
#' @param df A data frame or matrix of strictly positive relative frequencies or
#'   counts (samples/observations as columns, components as rows).
#' @return A data frame of the same dimensions containing the CLR-transformed
#'   values.
#' @export calc_clr
#' @examples
#' freq_imp <- data.frame(A = c(10.1, 89.9), B = c(50.1, 49.9))
#' calc_clr(freq_imp)
calc_clr <- function(df) {
    if (any(df <= 0, na.rm = TRUE)) {
        stop(
            "Data must be strictly positive for CLR. ",
            "Did you forget zero-imputation?"
        )
    }
    if (!all(vapply(df, is.numeric, logical(1)))) {
        stop("All columns in the input data must be numeric.")
    }

    log_df <- log(df)
    clr_df <- sweep(log_df, 2, colMeans(log_df), "-")

    return(as.data.frame(clr_df))
}


#' Replace zero values in count or frequency data
#'
#' This function replaces zero values in a data frame or matrix to facilitate
#' downstream transformations that require strictly positive values, such as the
#' Centered Log-Ratio (CLR) transformation.
#'
#' @details The function provides two methods for handling zeros:
#' \itemize{
#'   \item \strong{counts:} Adds a fixed value (\code{pseudo_count}).
#'   \item \strong{frac_min:} Replaces zeros with a fraction
#'   (\code{pseudo_frac_min}) of the smallest observed non-zero value in
#'   the dataset.
#' }
#'
#' @param df A data frame or matrix where zeros need to be replaced.
#' @param rep_method Character string specifying the replacement strategy. One
#'   of \code{"counts"}, or \code{"frac_min"}.
#' @param pseudo_count Numeric (default: 0.5). Used if \code{rep_method =
#'   "counts"}.
#' @param pseudo_frac_min Numeric (default: 2/3). The multiplier for the minimum
#'   non-zero value when \code{rep_method = "frac_min"}.
#' @param add_to Character string. Should the value be added to \code{"all"}
#'   cells or just the \code{"zeros"}?
#'
#' @return A data frame or matrix of the same dimensions as \code{df} with all
#'   zero values replaced by the calculated replacement value.
#'
#' @export replace_zeros
#'
#' @examples
#' # Replace zeros in a count matrix
#' counts_df <- data.frame(A = c(10, 0, 5), B = c(20, 10, 0))
#' replace_zeros(counts_df, rep_method = "counts", add_to = "all")
#'
#' # Replace zeros in a frequency matrix
#' freq_df <- data.frame(A = c(0.5, 0.5), B = c(0.2, 0.8), C = c(0.0, 1.0))
#' replace_zeros(freq_df, rep_method = "frac_min", add_to = "zeros")
replace_zeros <- function(df,
                          rep_method = c("counts", "frac_min"),
                          pseudo_count = 0.5,
                          pseudo_frac_min = 2 / 3,
                          add_to = NULL) {
    rep_method <- match.arg(rep_method)

    if (!all(vapply(df, is.numeric, logical(1)))) {
        stop("Input data must be numeric to perform zero replacement.")
    }

    if (is.null(add_to)) {
        add_to <- if (rep_method == "counts") "all" else "zeros"
    } else {
        add_to <- match.arg(add_to, c("all", "zeros"))
    }

    imp_val <- switch(rep_method,
        "counts"   = pseudo_count,
        "frac_min" = min(df[df > 0], na.rm = TRUE) * pseudo_frac_min
    )

    if (add_to == "all") {
        df <- df + imp_val
    } else {
        df[df == 0] <- imp_val
    }

    return(df)
}


#' Get the cell type counts from a long data frame (e.g. seurat object metadata)
#' where each cell is a row.
#'
#' @param cell_data_df A data frame where each row represents a single cell, and
#'   columns contain cell-level information, including sample ID and cell type
#'   annotation.
#' @param sample_col The column that defines the sample ID for each cell
#' @param celltype_col The column that defines the cell type annotation for each
#'   cell
#'
#' @return A data frame with cell types as rows and samples as columns,
#'   containing the count of each cell type per sample.
#' @export get_celltype_counts
#' @examples
#' # Create example data frame
#' cell_data_df <- data.frame(
#'     Cell_ID = paste0("C", seq(10)),
#'     Sample_Name = c(rep("S1", 5), rep("S2", 5)),
#'     Cluster_Annotation = factor(c(
#'         "B_Cell", "T_Cell", "B_Cell", NA, "T_Cell",
#'         "T_Cell", "Macrophage", "B_Cell", "T_Cell", "T_Cell"
#'     ))
#' )
#' #
#' # Calculate cell type counts per sample
#' celltype_counts <- get_celltype_counts(
#'     cell_data_df = cell_data_df,
#'     sample_col = "Sample_Name",
#'     celltype_col = "Cluster_Annotation"
#' )
#' #
#' print(celltype_counts)
#' # Note how the NA cell type count is handled and renamed to "NA".
get_celltype_counts <- function(cell_data_df,
                                sample_col,
                                celltype_col) {
    cellcount_df <- table(
        cell_data_df[[celltype_col]],
        cell_data_df[[sample_col]],
        useNA = "ifany"
    ) %>%
        as.data.frame.matrix()

    return(cellcount_df)
}


#' Extracts constant metadata for each sample from a cell-level data frame.
#'
#' This function identifies columns in a cell-level metadata data frame that
#' have a **constant** value for all cells belonging to the same sample. It
#' aggregates this constant information, returning a new data frame where each
#' row represents a unique sample. Columns that vary within any sample are
#' excluded from the output.
#'
#' @param cell_data_df A data frame containing cell-level metadata. This should
#'   include the sample ID column and all potential metadata columns.
#' @param sample_col A character string specifying the name of the column that
#'   defines the unique sample ID for each cell (e.g., "Sample_ID").
#' @return A new data frame where:
#'         \itemize{
#'           \item Each row corresponds to a unique sample from the input data.
#'           \item The row names are set to the values of the input
#'                 `sample_col`.
#'           \item Columns contain only the metadata fields that were constant
#'                 across all cells within **each** sample. The `sample_col`
#'                 itself is excluded from the final columns but used for
#'                 row names.
#'         }
#' @importFrom dplyr group_by summarise across everything n_distinct ungroup
#'   select where all_of distinct %>%
#' @importFrom rlang sym
#' @export get_sample_metadata
#' @examples
#' # Assuming you have a data frame 'cell_df'
#' cell_df <- data.frame(
#'     Cell_ID = paste0("C", seq(10)),
#'     Sample_ID = c(rep("S1", 5), rep("S2", 5)),
#'     Age = c(rep(30, 5), rep(45, 5)),
#'     Gender = c(rep("M", 5), rep("F", 5)),
#'     Cell_Type = c(rep("A", 3), rep("B", 2), rep("A", 3), rep("B", 2))
#' )
#'
#' # The 'Age' and 'Gender' columns are constant within each sample (S1 and S2).
#' # The 'Cell_ID' and 'Cell_Type' columns vary within sample S1 and/or S2.
#'
#' sample_meta <- get_sample_metadata(cell_df, "Sample_ID")
#' print(sample_meta)
#' # Output will have 'Age' and 'Gender' as columns,
#' # with row names 'S1' and 'S2'.
get_sample_metadata <- function(cell_data_df,
                                sample_col) {
    # Step 1: Group the data by `sample_col` and check for uniqueness
    distinct_counts <- cell_data_df %>%
        group_by(!!sym(sample_col)) %>%
        summarise(across(everything(), ~ n_distinct(.x))) %>%
        ungroup()

    # Step 2: Identify columns that are constant across all samples
    constant_cols <- distinct_counts %>%
        select(where(~ max(.x) == 1)) %>%
        names()

    cols_to_keep <- unique(c(sample_col, constant_cols))

    # Step 3: Select the constant columns and unique rows
    metadata <- cell_data_df %>%
        # Select only the relevant constant columns
        select(all_of(cols_to_keep)) %>%
        distinct()

    rownames(metadata) <- metadata[[sample_col]]

    # Remove the sample column from the final metadata table
    cols_to_return <- setdiff(colnames(metadata), sample_col)

    metadata <- metadata[, cols_to_return, drop = FALSE] %>%
        as.data.frame()

    return(metadata)
}


# Find highly variable cell types ---------------------------

#' Identifies and stores Highly Variable Cell Types (HVCs) in an
#' SummarizedExperiment object.
#'
#' This function calculates the variance of each cell type across all samples
#' based on the Centered Log-Ratio (CLR) transformed data. It then selects the
#' subset of highly variable cell types (HVCs) that collectively explain a
#' specified proportion of the total variance, or a fixed number of top cell
#' types.
#'
#' @param se An initialized
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object containing the CLR-transformed data in the \code{clr} assay.
#' @param variance_explained Numeric (default: 0.5). The target cumulative
#'   proportion of total variance to be explained by the selected HVCs. The
#'   function stops selecting cell types once this threshold is met.
#' @param top_n_hvcs Integer (optional). If provided, this overrides
#'   \code{variance_explained} and selects exactly the top N cell types with the
#'   highest variance.
#'
#' @importFrom S4Vectors metadata metadata<-
#'
#' @return The updated
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object with the following metadata populated:
#'         \itemize{
#'           \item \code{celltype_variances}: Data frame of cell type variances.
#'           \item \code{variance_explained}: The exact variance proportion
#'                 captured by the selected HVCs.
#'           \item \code{top_n_hvcs}: The number of HVCs selected.
#'           \item \code{hvcs}: Character vector of the names
#'                 of the selected HVCs.
#'         }
#'
#' @export find_hvcs
#'
#' @seealso
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment},
#'   \code{\link{get_celltype_variances}}, \code{\link{get_hvcs}}
#'
#' @examples
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#'
#' # Select HVCs that explain 75% of the total variance
#' se <- find_hvcs(se, variance_explained = 0.75)
#'
#' # Select exactly the top 5 most variable cell types
#' se <- find_hvcs(se, top_n_hvcs = 5)
find_hvcs <- function(se,
                      variance_explained = 0.5,
                      top_n_hvcs = NULL) {
    stopifnot(is.numeric(variance_explained) &&
        variance_explained > 0 && variance_explained <= 1)

    if (!is.null(top_n_hvcs)) {
        stopifnot(is.numeric(top_n_hvcs) && top_n_hvcs > 0)
    }

    df_var <- get_celltype_variances(se)

    hvcs <- get_hvcs(
        df_var,
        variance_explained,
        top_n_hvcs = top_n_hvcs
    )

    variance_explained <- (sum(df_var$Variance[seq_along(hvcs)]) /
        sum(df_var$Variance))

    metadata(se)$celltype_variances <- df_var
    metadata(se)$variance_explained <- variance_explained
    metadata(se)$top_n_hvcs <- length(hvcs)
    metadata(se)$hvcs <- hvcs


    return(se)
}


#' Calculates the variance of cell types across samples.
#'
#' This function takes the Centered Log-Ratio (CLR) transformed cell type
#' abundance data from an
#' \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#' object, calculates the mean CLR abundance and variance for each cell type. It
#' also calculates the cumulative variance explained by the cell types when
#' ranked by variance.
#'
#' @param se An initialized
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object containing CLR-transformed data in the \code{clr} assay
#' @param descending Logical (default: \code{TRUE}). If \code{TRUE}, the
#'   returned data frame is sorted by variance in descending order.
#'
#' @return A data frame where each row is a cell type, containing:
#'         \itemize{
#'           \item \code{celltype}: The name of the cell type.
#'           \item \code{avg_clr_abundance}: The mean CLR value for that
#'                 cell type across all samples.
#'           \item \code{Variance}: The variance of the CLR values for that
#'                 cell type across all samples.
#'           \item \code{cumulative_variance}: Cumulative sum of the variance,
#'                 ranked by variance.
#'           \item \code{variance_exp}: The proportion of total variance
#'                 explained cumulatively.
#'         }
#'
#' @importFrom stats var
#' @importFrom dplyr %>% everything group_by summarize arrange desc mutate
#' @importFrom tidyr pivot_longer
#' @importFrom SummarizedExperiment assay
#'
#' @export get_celltype_variances
#'
#' @seealso
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment},
#'   \code{\link{plot_varmean}}
#'
#' @examples
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#'
#' # Calculate variances without plotting, sorted by ascending variance
#' df_var_asc <- get_celltype_variances(
#'     se,
#'     descending = FALSE
#' )
get_celltype_variances <- function(se, descending = TRUE) {
    clr_data <- assay(se, "clr")

    df_var <- data.frame(
        celltype = rownames(clr_data),
        avg_clr_abundance = rowMeans(clr_data, na.rm = TRUE),
        Variance = apply(clr_data, 1, var, na.rm = TRUE)
    )

    if (descending) {
        df_var <- df_var[order(-df_var$Variance), , drop = FALSE]
    } else {
        df_var <- df_var[order(df_var$Variance), , drop = FALSE]
    }

    total_variance <- sum(df_var$Variance, na.rm = TRUE)
    df_var$cumulative_variance <- cumsum(df_var$Variance)
    df_var$variance_exp <- df_var$cumulative_variance / total_variance

    rownames(df_var) <- NULL

    return(df_var)
}


#' Selects Highly Variable Cell Types (HVCs) based on variance or count
#' threshold.
#'
#' This is a utility function that selects the most variable cell types from a
#' variance-ranked data frame. Selection can be controlled either by defining
#' the cumulative variance to be explained or by specifying a fixed number of
#' cell types.
#'
#' @param df_var A data frame (typically the output of
#'   \code{\link{get_celltype_variances}}) that must contain at least the
#'   columns \code{celltype} and \code{variance_exp} (cumulative variance
#'   explained) and must be sorted by variance in descending order.
#' @param variance_explained Numeric (default: 0.5). The cumulative proportion
#'   of total variance that the selected HVCs must explain. This parameter is
#'   ignored if \code{top_n_hvcs} is set.
#' @param top_n_hvcs Integer or Numeric (optional).
#'                   \itemize{
#'                     \item If an integer (>= 1), it specifies the exact
#'                           number of top cell types to select.
#'                     \item If a numeric value between 0 and 1, it specifies
#'                           the proportion of cell types to select
#'                           (e.g., 0.1 for the top 10%).
#'                   }
#'
#' @return A character vector containing the names of the selected highly
#'   variable cell types. The function ensures that at least two cell types are
#'   always returned.
#'
#' @importFrom dplyr %>% slice pull slice_head
#' @importFrom rlang .data
#'
#' @export get_hvcs
#'
#' @seealso \code{\link{find_hvcs}}, \code{\link{get_celltype_variances}}
#'
#' @examples
#' data(example_data)
#' ecoda_object <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#'
#' # Calculate variances
#' df_var <- get_celltype_variances(ecoda_object)
#'
#' # Select cell types explaining at least 60% of variance:
#' hvcs_60 <- get_hvcs(df_var, variance_explained = 0.6)
#'
#' # Select the top 10 cell types:
#' hvcs_top10 <- get_hvcs(df_var, top_n_hvcs = 10)
#'
#' # Select the top 5% of cell types:
#' hvcs_prop <- get_hvcs(df_var, top_n_hvcs = 0.05)
get_hvcs <- function(df_var,
                     variance_explained = 0.5,
                     top_n_hvcs = NULL) {
    if (!is.null(top_n_hvcs)) {
        message("Setting top_n_hvcs overrules variance_explained parameter")
        if (top_n_hvcs < 1) {
            top_hvcs <- df_var %>%
                slice_head(prop = top_n_hvcs) %>%
                pull(.data$celltype)
        } else {
            top_hvcs <- df_var %>%
                slice_head(n = top_n_hvcs) %>%
                pull(.data$celltype)
        }
    } else {
        last_hvc_index <- which.max(df_var$variance_exp >= variance_explained)

        if (length(last_hvc_index) == 0 || last_hvc_index == 0) {
            warning(
                "Variance threshold was not met for any cell type ",
                "(or data is empty). Returning top 2."
            )
            return(df_var$celltype[seq_len(min(2, nrow(df_var)))])
        }

        top_hvcs <- df_var %>%
            slice(seq_len(last_hvc_index)) %>%
            pull(.data$celltype)
    }

    # Select at least two cell types
    if (length(top_hvcs) < 2) {
        message("The minimum number of highly variable cell types to use is 2")
        top_hvcs <- df_var %>%
            slice_head(n = 2) %>%
            pull(.data$celltype)
    }

    return(top_hvcs)
}


#' @title Generates a Mean-Variance Plot for CLR-transformed Cell Type Data.
#'
#' @description This function visualizes the relationship between the mean
#'   abundance (CLR) and the variance (CLR) for each cell type, which is
#'   typically used to identify Highly Variable Cell Types (HVCs).
#'
#' @details If \code{highlight_hvcs} is \code{TRUE}, cell types previously
#'   identified and stored in \code{metadata(se)$hvcs} will be highlighted in
#'   red on the plot.
#'
#' @param se A
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
#'   object containing pre-calculated cell type variances in
#'   \code{metadata(se)$celltype_variances} and the HVC list in
#'   \code{metadata(se)$hvcs}
#' @param plot_title Character string (default: ""). The title for the plot.
#' @param highlight_hvcs Logical (default: \code{TRUE}). If \code{TRUE}, the
#'   points corresponding to the Highly Variable Cell Types (HVCs) stored in the
#'   SummarizedExperiment object are colored red.
#' @param labels Character (default: "only_hvc"). Options: "all" (label every
#'   point), "none" (no labels), or "only_hvc" (label only the highly variable
#'   cell types).
#' @param plot_fit_line Logical (default: \code{FALSE}). If \code{TRUE}, a
#'   smoothing regression line is added to the plot.
#' @param smooth_method Character string (default: "lm"). The smoothing method
#'   to use for the regression line if \code{plot_fit_line} is \code{TRUE}
#'   (e.g., "lm" for linear model, "loess" for local regression).
#'
#' @return A \code{ggplot} object representing the mean-variance plot.
#'
#' @importFrom ggplot2 ggplot aes geom_point geom_smooth labs theme_classic xlab
#'   ylab scale_color_manual theme
#' @importFrom ggrepel geom_text_repel
#' @importFrom dplyr mutate if_else filter
#' @importFrom S4Vectors metadata
#'
#' @export plot_varmean
#'
#' @seealso
#'   \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment},
#'   \code{\link{get_celltype_variances}}
#'
#' @examples
#' data(example_data)
#' se <- ecoda(
#'     data = example_data$Gongsharma_full$cell_counts$authors_HR,
#'     metadata = example_data$Gongsharma_full$metadata
#' )
#'
#' # 1. Generate the plot, highlighting HVCs (default)
#' plot_varmean(se, plot_title = "HVCs on Mean-Variance Plot")
#'
#' # 2. Generate the plot without highlighting HVCs and add a fit line
#' plot_varmean(se,
#'     highlight_hvcs = FALSE,
#'     plot_fit_line = TRUE,
#'     smooth_method = "loess"
#' )
plot_varmean <- function(se,
                         plot_title = "",
                         highlight_hvcs = TRUE,
                         labels = c("only_hvc", "all", "none"),
                         plot_fit_line = FALSE,
                         smooth_method = "lm") {
    labels <- match.arg(labels)
    df_var <- metadata(se)$celltype_variances
    stopifnot(is.character(plot_title) || is.null(plot_title))
    stopifnot(is.logical(highlight_hvcs))
    stopifnot(is.logical(plot_fit_line))

    highlight_celltypes <- metadata(se)$hvcs

    # --- 1. Create a highlighting factor column ---
    if (highlight_hvcs) {
        df_var <- df_var %>%
            mutate(
                is_highlighted = if_else(
                    .data$celltype %in% highlight_celltypes,
                    "HVC selected",
                    "Not selected"
                )
            )
        color_map <- c("HVC selected" = "red", "Not selected" = "black")

        p <- ggplot(
            df_var,
            aes(
                x = .data$avg_clr_abundance,
                y = .data$Variance,
                color = .data$is_highlighted
            )
        ) +
            scale_color_manual(values = color_map, name = "Cell Type Group")
    } else {
        p <- ggplot(
            df_var, aes(x = .data$avg_clr_abundance, y = .data$Variance)
        )
    }

    p <- p +
        geom_point() +
        labs(title = paste(plot_title)) +
        theme_classic() +
        xlab("Mean (CLR)") +
        ylab("Variance (CLR)")

    if (plot_fit_line) {
        p <- p +
            geom_smooth(
                method = smooth_method,
                color = "red",
                fill = "#69b3a2",
                se = TRUE
            )
    }

    # Ensure the legend is not shown if we are not highlighting anything
    if (!highlight_hvcs) p <- p + theme(legend.position = "none")

    if (labels != "none") {
        # Filter data for labeling based on user choice
        label_df <- df_var
        if (labels == "only_hvc") {
            label_df <- df_var %>%
                dplyr::filter(.data$celltype %in% highlight_celltypes)
        }

        # Use the color aesthetic inside ggrepel so
        # highlighted labels match the point color
        if (highlight_hvcs) {
            p <- p +
                geom_text_repel(
                    data = label_df,
                    aes(label = .data$celltype, color = .data$is_highlighted),
                    size = 3,
                    show.legend = FALSE
                )
        } else {
            p <- p +
                geom_text_repel(
                    data = label_df,
                    aes(label = .data$celltype),
                    size = 3,
                    color = "black"
                )
        }
    }

    return(p)
}


# Get Pseudobulk and normalize ---------------------------

#' Calculate Pseudobulk from Count Matrix
#'
#' This function aggregates single-cell count data into a "pseudobulk" matrix by
#' summing the counts for all cells belonging to the same sample ID. It is
#' robust to both dense and sparse count matrices. It also includes filtering
#' logic to exclude samples that do not meet a minimum cell count threshold.
#'
#' @param count_matrix A gene x cell count matrix (can be a dense matrix or
#'   sparse Matrix). Gene identifiers should be row names and cell barcodes
#'   should be column names.
#' @param sample_ids A vector of sample identifiers, one for each column (cell)
#'   in \code{count_matrix}. The length must equal \code{ncol(count_matrix)}.
#'   Must not contain \code{NA} values.
#' @param min_cells Minimum number of cells required per sample (default: 1).
#'   Samples with fewer cells than this threshold will be excluded from the
#'   final pseudobulk matrix.
#'
#' @return A gene x sample pseudobulk count matrix. The columns correspond to
#'   the unique sample IDs, and the rows correspond to the genes.
#' @export calculate_pseudobulk
#' @examples
#' nrow <- 100
#' ncol <- 100
#' vals <- nrow * ncol
#' mat <- round(matrix(exp(rpois(vals, lambda = 3)), nrow = nrow, ncol = ncol))
#' rownames(mat) <- paste0("Gene", seq_len(ncol))
#' colnames(mat) <- paste0("Cell", seq_len(nrow))
#' ids <- rep(c("Sample1", "Sample2"), each = nrow / 2)
#' pb <- calculate_pseudobulk(count_matrix = mat, sample_ids = ids)
calculate_pseudobulk <- function(count_matrix,
                                 sample_ids,
                                 min_cells = 10) {
    # Input validation
    if (ncol(count_matrix) != length(sample_ids)) {
        stop(
            "Length of sample_ids must equal ",
            "the number of columns in count_matrix"
        )
    }

    if (any(is.na(sample_ids))) {
        stop(
            "sample_ids contains NA values. ",
            "Please remove or replace missing values."
        )
    }

    stopifnot(is.numeric(min_cells) && min_cells >= 1)

    # Convert sample_ids to factor for grouping
    sample_ids <- as.factor(sample_ids)

    # Count cells per sample
    cells_per_sample <- table(sample_ids)

    # Filter samples with insufficient cells
    if (min_cells > 1) {
        valid_samples <- names(cells_per_sample)[cells_per_sample >= min_cells]

        if (length(valid_samples) == 0) {
            stop("No samples have >= ", min_cells, "cells")
        }

        # Subset to valid samples
        keep_cells <- sample_ids %in% valid_samples
        count_matrix <- as.matrix(count_matrix)[, keep_cells, drop = FALSE]
        sample_ids <- droplevels(sample_ids[keep_cells])

        # Report filtered samples
        n_filtered <- length(cells_per_sample) - length(valid_samples)
        if (n_filtered > 0) {
            message(
                "Filtered out ", n_filtered,
                " sample(s) with < ", min_cells, " cells"
            )
        }
    }

    # Aggregate by summing across samples
    pb <- t(rowsum(t(count_matrix), group = sample_ids))

    return(pb)
}


#' DESeq2 Normalization of Pseudobulk Data
#'
#' This function normalizes a gene x sample pseudobulk count matrix using the
#' Variance Stabilizing Transformation (VST) from the \code{DESeq2} package. It
#' estimates size factors and variance for all genes, performs the VST, and then
#' subsets the results to include only highly variable genes (HVCs), either
#' specified by the user or automatically selected based on variance.
#'
#' **Note:** If `deseq2_design` is not specified, the VST is performed using a
#' minimal design formula (`~ 1`) for size factor and variance estimation.
#'
#' @param pb A gene x sample pseudobulk count matrix (Genes as rows, Samples as
#'   columns). Must contain non-negative integer counts.
#' @param metadata A data.frame with sample-level metadata. Row names must
#'   exactly match the column names of \code{pb}. The function will reorder the
#'   metadata to match \code{pb}.
#' @param deseq2_design Optional: A \code{formula} object specifying the design
#'   used by \code{DESeq2} to estimate size factors and variance for the VST.
#'   This formula should reference columns in the \code{metadata} data frame. If
#'   \code{NULL} (the default), the minimal design \code{~ 1} is used. Note: the
#'   design will be considered for pseudobulk normalization as the function uses
#'   vst(blind = FALSE, ...) internally.
#' @param hvg Optional character vector of gene names to use as highly variable
#'   genes. If provided, only these genes will be returned after VST.
#' @param nvar_genes Number of top variable genes to select (default: 2000).
#'   Only used if \code{hvg = NULL}; the function will select the top
#'   \code{nvar_genes} by variance after VST.
#'
#' @return A normalized expression matrix (VST-transformed) with\
#'         **samples as columns** and **genes as rows**.
#'
#' @export deseq2_normalize
#'
#' @importFrom stats formula
#' @importFrom DESeq2 DESeqDataSetFromMatrix estimateSizeFactors vst
#' @importFrom BiocGenerics counts
#' @importFrom SummarizedExperiment assay
#'
#' @examples
#' nrow <- 100
#' ncol <- 100
#' vals <- nrow * ncol
#' mat <- round(matrix(exp(rpois(vals, lambda = 3)), nrow = nrow, ncol = ncol))
#' rownames(mat) <- paste0("Gene", seq_len(ncol))
#' colnames(mat) <- paste0("Cell", seq_len(nrow))
#' ids <- rep(c("Sample1", "Sample2"), each = nrow / 2)
#' pb <- calculate_pseudobulk(count_matrix = mat, sample_ids = ids)
#' pb_norm <- deseq2_normalize(pb)
deseq2_normalize <- function(pb,
                             metadata = NULL,
                             deseq2_design = NULL,
                             hvg = NULL,
                             nvar_genes = 2000) {
    stopifnot(is.numeric(nvar_genes) && nvar_genes >= 1)

    if (is.null(metadata)) {
        metadata <- data.frame(
            # Add dummy column required for the design formula (~ 1)
            dummy_group = factor(rep("A", ncol(pb))),
            # The row names MUST match the sample names in the count matrix (pb)
            row.names = colnames(pb)
        )
    } else {
        # --- Existing check/reorder for when metadata IS provided ---
        if (!all(colnames(pb) %in% rownames(metadata))) {
            stop("Not all sample names in pb are found in metadata rownames")
        }
        # Reorder metadata to match pb
        metadata <- metadata[colnames(pb), , drop = FALSE]
    }

    if (is.null(deseq2_design)) deseq2_design <- formula(paste("~ 1"))

    # Create DESeq2 dataset (metadata is guaranteed to exist here)
    dds <- DESeqDataSetFromMatrix(
        countData = pb,
        colData = metadata,
        design = deseq2_design
    )

    # Estimate size factors
    dds <- estimateSizeFactors(dds)

    # Set minimum number of counts per gene for VST
    nsub <- min(1000, sum(rowMeans(counts(dds, normalized = TRUE)) > 10))

    # Transform counts using variance stabilizing transformation (VST)
    dds <- vst(dds, blind = FALSE, nsub = nsub)
    pb_norm <- assay(dds) # Genes x Samples format

    # Select highly variable genes
    if (!is.null(hvg)) {
        # Use user-provided HVGs
        message("Using", length(hvg), "user-provided highly variable genes")

        hvg_available <- hvg[hvg %in% rownames(pb_norm)]
        hvg_missing <- setdiff(hvg, hvg_available)

        if (length(hvg_missing) > 0) {
            warning(
                length(hvg_missing),
                " genes not found in the data and will be excluded"
            )
        }

        if (length(hvg_available) == 0) {
            stop("None of the provided HVGs are found in the data")
        }

        pb_norm <- pb_norm[hvg_available, , drop = FALSE]
    } else {
        # Auto-select top variable genes
        rv <- apply(pb_norm, 1, var)
        n_genes_to_select <- min(nvar_genes, length(rv))
        select <- order(rv, decreasing = TRUE)[seq_len(n_genes_to_select)]
        select <- rownames(pb_norm)[select]
        pb_norm <- pb_norm[select, , drop = FALSE]

        message("Selected top ", nrow(pb_norm), " highly variable genes")
    }

    return(as.data.frame(pb_norm))
}

test_that("Frequency calculation is accurate", {
    # Test that rows sum to exactly 100
    counts <- data.frame(A = c(10, 50), B = c(90, 50))
    freq <- calc_freq(counts)

    expect_equal(colSums(freq), c(A = 100, B = 100))
    expect_s3_class(freq, "data.frame")
})

test_that("Zero imputation handles counts and frequencies correctly", {
    # Test count imputation (multiplicative replacement)
    counts <- data.frame(A = c(10, 0), B = c(20, 10))
    counts_imp <- replace_zeros(counts, rep_method = "counts")

    expect_false(any(counts_imp == 0))
    expect_equal(counts_imp[2, 1], 0.5) # Default xmin_factor * counts_min

    # Test frequency imputation (fraction of smallest non-zero)
    freq <- data.frame(A = c(0.5, 0), B = c(0.5, 1.0))
    freq_imp <- replace_zeros(freq, rep_method = "frac_min")

    expect_false(any(freq_imp == 0))
    expect_equal(freq_imp[2, 1], 0.5 * (2 / 3))
})

test_that("CLR transformation is mathematically consistent", {
    # CLR should fail or warn if zeros are present (based on your main.r code)
    # clr() function in main.r assumes positive data.
    pos_data <- data.frame(A = c(10, 5), B = c(20, 25))
    clr_res <- calc_clr(pos_data)

    expect_equal(nrow(clr_res), 2)
    expect_equal(ncol(clr_res), 2)

    # Mathematical property: row sums of CLR-transformed data should be 0
    expect_equal(colSums(clr_res), c(A = 0, B = 0), tolerance = 1e-9)
})

test_that("get_celltype_variances calculates metrics correctly", {
    # Create an ECODA object with known variance
    # Celltype 1 varies a lot, Celltypes 2, 3 and 4 vary less
    df <- data.frame(
        A = c(10, 20, 5, 400),
        B = c(11, 30, 10, 250),
        C = c(12, 40, 15, 130)
    )
    rownames(df) <- c("ct1", "ct2", "ct3", "ct4")

    se <- ecoda(data = df)

    var_df <- get_celltype_variances(se)

    expect_equal(var_df$celltype[1], "ct4")
    expect_true(var_df$Variance[1] > var_df$Variance[2])
    expect_equal(var_df$variance_exp[nrow(var_df)], 1.0) # Cumulative should reach 1
})

test_that("get_hvcs selection logic is robust", {
    df_var <- data.frame(
        celltype = paste0("C", 1:10),
        Variance = 10:1,
        variance_exp = cumsum(10:1) / sum(10:1)
    )

    # Test top_n_hvcs (Integer)
    hvcs_n <- get_hvcs(df_var, top_n_hvcs = 3)
    expect_length(hvcs_n, 3)
    expect_equal(hvcs_n, c("C1", "C2", "C3"))

    # Test variance_explained threshold
    # C1 is ~0.18, C2 is ~0.34...
    hvcs_var <- get_hvcs(df_var, variance_explained = 0.5)
    expect_true(length(hvcs_var) >= 2)

    # Test minimum return (always at least 2)
    hvcs_min <- get_hvcs(df_var[1:5, ], top_n_hvcs = 1)
    expect_length(hvcs_min, 2)
})

test_that("get_celltype_counts handles NA and dimensions correctly", {
    cell_df <- data.frame(
        Sample = c(rep("S1", 3), rep("S2", 2)),
        Type = c("B", "T", NA, "T", "T")
    )

    counts <- get_celltype_counts(
        cell_data_df = cell_df,
        sample_col = "Sample",
        celltype_col = "Type"
    )

    # Check dimensions: 2 samples, 3 categories (B, T, and NA)
    expect_equal(ncol(counts), 2)
    expect_equal(nrow(counts), 3)
    expect_true("NA." %in% rownames(counts))
    expect_equal(counts["NA.", "S1"], 1)
})

test_that("get_sample_metadata filters non-constant columns", {
    cell_df <- data.frame(
        Sample_ID = c(rep("S1", 3), rep("S2", 2)),
        Age = c(30, 30, 30, 45, 45), # Constant per sample
        Cell_ID = paste0("C", 1:5), # Varies per sample
        Condition = c("A", "A", "A", "B", "B") # Constant per sample
    )

    sample_meta <- get_sample_metadata(cell_df, "Sample_ID")

    # Should keep Age and Condition, but drop Cell_ID
    expect_true(all(c("Age", "Condition") %in% colnames(sample_meta)))
    expect_false("Cell_ID" %in% colnames(sample_meta))
    expect_equal(nrow(sample_meta), 2)
    expect_equal(rownames(sample_meta), c("S1", "S2"))
})

test_that("ECODA SingleCellExperiment object constructor works", {
    data(example_data)
    d <- example_data$Zhang

    se <- ecoda(data = d$cell_counts$authors_LR, metadata = d$metadata)

    expect_s4_class(se, "SummarizedExperiment")
    expect_equal(ncol(assay(se, "clr")), nrow(colData(se)))
    expect_true(all(assay(se, "freq_imp") > 0))
})

test_that("calculate_pseudobulk aggregates and filters correctly", {
    # 3 genes, 6 cells
    counts <- matrix(1, nrow = 3, ncol = 6)
    rownames(counts) <- c("G1", "G2", "G3")
    colnames(counts) <- paste0("Cell", 1:6)
    samples <- c("S1", "S1", "S1", "S1", "S2", "S2")

    # Basic sum
    pb <- calculate_pseudobulk(counts, samples, min_cells = 1)
    expect_equal(pb["G1", "S1"], 4)
    expect_equal(ncol(pb), 2)

    # Test min_cells filtering
    # S2 only has 2 cells, so if we set min_cells to 4, it should drop S2
    expect_message(
        pb_filtered <- calculate_pseudobulk(counts, samples, min_cells = 4),
        "Filtered out 1 sample\\(s\\) with < 4 cells"
    )
    expect_equal(ncol(pb_filtered), 1)
    expect_equal(colnames(pb_filtered), "S1")
})

#' Example Data for scECODA
#'
#' A collection of single-cell compositional data from multiple studies,
#' structured to demonstrate the full scECODA workflow.
#'
#' @format A named \code{list} where each element corresponds to a different
#'   study (e.g., \code{$Adams}). Each study element is a \code{list} containing
#'   the following:
#' \itemize{
#'    \item \code{cell_counts_highresolution}: Data frame of cell type counts at
#'    high resolution annotation (samples as rows, cell types as columns).
#'    \item \code{cell_counts_lowresolution}: Data frame of cell type counts at
#'    low resolution annotation (samples as rows, cell types as columns).
#'    \item \code{metadata}: Data frame of sample-level metadata (samples as
#'    rows).
#'    \item \code{main_biologicalcondition_columnname}: Character string,
#'    indicating the main biological condition column name in the metadata.
#' }
#' @usage data(example_data)
#' @source Data originally derived from multiple public single-cell datasets and
#'   re-processed as described in the accompanying scECODA manuscript (Halter et
#'   al., *in preparation*).
"example_data"

---
title: "scECODA tutorial"
author: "Christian Halter"
date: "`r Sys.Date()`"
output:
  BiocStyle::html_document:
    toc_float: true
  BiocStyle::pdf_document: default
vignette: >
  %\VignetteIndexEntry{scECODA tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r Load libraries}
library(scECODA)
library(ggplot2)

# Load example datasets
data(example_data)
```


```{r  Example, fig.height=7, fig.width=9}
ds <- example_data$GongSharma_full
counts <- ds$cell_counts_highresolution
metadata <- ds$metadata
metadata$age_cmv <- paste0(metadata$age_group, " CMV-", metadata$subject.cmv)

ecoda_object <- ecoda(
    data = counts,
    metadata = metadata
)

p <- plot_pca(
    ecoda_object,
    label_col = "age_cmv",
    title = paste0(
        "PCA - Population-scale sample representation\n",
        "Based on cell type composition"
    ),
    n_hv_feat_show = 9,
    pointsize = 1.75,
    labelsize = 3,
    cluster_score = FALSE, mod_score = FALSE, anosim_score = FALSE
) +
    guides(shape = guide_legend(override.aes = list(size = 3.3))) +
    theme(
        # axis.text=element_text(size=14),
        legend.title = element_text(size = 14),
        legend.text = element_text(size = 12)
    )
print(p)

out_dir <- file.path(".", "man", "figures")
if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)
ggsave(
  filename = file.path(out_dir, "GongSharma.png"),
  plot = p,
  height = 7,
  width = 9
)
```

```{r Session Info}
sessionInfo()
```

---
title: "scECODA tutorial"
author: "Christian Halter"
date: "`r Sys.Date()`"
output:
  BiocStyle::html_document:
    toc_float: true
  BiocStyle::pdf_document: default
editor_options: 
  markdown: 
    wrap: 80
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{scECODA tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
# Solution for the "magick package is required to crop" error:
knitr::opts_chunk$set(
  echo = TRUE, # keep showing code chunks
  message = FALSE, 
  warning = FALSE,
  # This is the line that fixes the cropping issue:
  crop = NULL
)
```

# Introduction

The scECODA package provides functions for *s*ingle-*c*ell *E*xploratory
*CO*mpositional *D*ata *A*nalysis at the population scale. It enables users to
explore their dataset by visualizing the sample structure and how groups of
samples relate to each other by providing numerous convenient plotting and
clustering functions. Additionally, it provides metrics to quantify how strongly
groups of samples separate and which cell types are driving this separation
(differential abundance analysis).

## Motivation for Bioconductor Integration

While many tools exist for supervised comparative single-cell analysis, there is
a distinct need for a standardized framework that handles unsupervised
**exploratory compositional data analysis (ECODA)** specifically for large-scale
cohorts. `scECODA` is motivated by the following goals for the Bioconductor
ecosystem:

-   **Interoperability**: The package natively supports core Bioconductor
    classes, including `SingleCellExperiment` and `SummarizedExperiment`,
    allowing seamless integration into existing pipelines.
-   **Infrastructure Synergy**: It leverages the established Bioconductor tool
    `DESeq2` for pseudobulk normalization.

--------------------------------------------------------------------------------

# Installation

You can install the latest stable release of `scECODA` from Bioconductor.

```{r Install the scECODA package, eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("scECODA")
```

```{r Load scECODA}
library(scECODA)
```

--------------------------------------------------------------------------------

# Create SummarizedExperiment objects for ECODA

You can create an SummarizedExperiment object from a single-cell data (Seurat or
SingleCellExperiment object) or from a data frame containing cell counts or
frequencies (in % per sample) with `ecoda()`.

Based on the input, the following data and transformations will be automatically
calculated:

-   Cell counts per sample ("counts")
-   Relative abundance in % ("freq")
-   Centered log-ratio (CLR) transformed abundance ("clr")
-   Arc-sine square root transformed abundance ("asin_sqrt")
-   Optionally: you can get the pseudobulk gene expression per sample,
    normalized with DESeq2 ("pb")
-   Metadata

Additionally, it will calculate useful metrics, such as:

-   The variance of each cell type across all samples (in CLR value) in a data
    frame. It also contains their average abundance (in CLR) and cumulative
    variance explained.
-   The top *n* most highly variable cell types (HVCs) ("top_n_hvcs"). The
    number of HVCs (*n*) is automatically determined by selecting the most
    highly variable cell types explaining approximately 50% of the total
    variance (sum of variances of all cell types). Alternatively, the user can
    choose the number of cell types *n* or a variance explained threshold.
-   The names of the top *n* most highly variable cell types (HVCs) ("hvcs")
-   The variance explained by the top_n_hvcs ("variance_explained")
-   Sample distance matrix ("sample_distances")

**NOTE:** To calculate CLR-transformed cell type abundances, zero counts or
relative abundances are not allowed (log of zero is not defined). By default,
scECODA adds a pseudocount of +0.5 to all values if a counts matrix is provided;
when the input is a frequency matrix, we add 2/3 times the smallest frequency
value as suggested in [@martín-fernández2003].

## From SingleCellExperiment object

Load an example dataset from the `scRNAseq` BioC library.

```{r Load example SingleCellExperiment dataset, message=FALSE, warning=FALSE}
# Loading an example SingleCellExperiment dataset
library(scRNAseq)

all.ds <- as.data.frame(surveyDatasets())
ds <- 1
sce_object <- fetchDataset(
  all.ds[["name"]][ds],
  all.ds[["version"]][ds]
)

names(colData(sce_object))
```

To represent this dataset in terms of cell type composition, we will use the
cell type labels provided by the authors and stored in the "cluster" metadata
field. We will do this for each individual sample ("sample" metadata field). The
`ecoda()` function will do this for you:

```{r Create ECODA object}
# Create ECODA object
se <- ecoda(
  sce_object,
  sample_col = "sample",
  celltype_col = "cluster"
)
```

The resulting object stores row counts, frequencies and several transformations
for the cell type composition by sample. For instance, the `clr` assay contains
centered log-ratio values for the cell types in each sample:

```{r Show example of CLR df}
assay(se, "clr")[1:5,1:5]
```

## From Seurat object

Alternatively, scECODA also supports generating compositional matrices from
Seurat objects:

```{r Create ECODA object from Seurat object}
# Commented out because Seurat depends on igraph which has not been updated
# to work with R 4.6 yet.

# library(Seurat)
# 
# counts <- counts(sce_object)
# metadata <- as.data.frame(SummarizedExperiment::colData(sce_object))
# 
# seurat_object <- CreateSeuratObject(counts = counts, meta.data = metadata)
# 
# se <- ecoda(
#   seurat_object,
#   sample_col = "sample",
#   celltype_col = "cluster"
# )
```

## From counts or frequency data frame

If you have a pre-calculated sample x cell type compositional matrix (either as
counts or raw frequencies), you can use them directly as input to scECODA.
Metadata can be optionally provided and used downstream for visualization.

In this example, we load the immune cell composition in samples from a breast
cancer cohort [@zhang2021]

**NOTES:**

-   If you create an SummarizedExperiment ECODA object from a data frame
    containing relative abundances (in %), please use percentage values (e.g. 30
    for 30%) and not as decimals (NOT 0.3 for 30%). If the relative abundances
    sum up to 100%, they will be automatically re-scaled to 100 and you will be
    informed with a message.
-   If you do add metadata, **it is important to make sure that the
    counts/frequency and the metadata data frames have the same column names**.

```{r Load example datasets}
# Load example datasets
data(example_data)

Zhang <- example_data$Zhang
counts <- Zhang$cell_counts$authors_LR
freq <- calc_freq(counts)
metadata <- Zhang$metadata

se <- ecoda(data = counts, metadata = metadata)

se <- ecoda(data = freq, metadata = metadata)
```

--------------------------------------------------------------------------------

# Visualization and quantification of sample separation

## PCA

For explorative analysis, Principal Component Analysis (PCA) can be very useful
to find the major axes of variation between samples. You can plot a PCA of your
se with `plot_pca(se)`.

```{r Show PCA based on cell type composition, message=FALSE, warning=FALSE}
se <- ecoda(
  data = example_data$Zhang$cell_counts$authors_LR,
  metadata = example_data$Zhang$metadata,
)

plot_pca(
  se,
  label_col = "Tissue",
  title = "PCA based on cell type composition",
  n_hv_feat_show = 5 # Shows the most highly variable features (cell types)
)
```

Not all cell types have informative composition (some may be very rare, others
constant across samples). It is often more robust to restrict the analysis on
highly-variable cell types (HVCs); for example, PCA can be calculated on HVCs:

```{r PCA based on highly variable cell types}
plot_pca(
  se,
  assay = "clr_hvc",
  label_col = "Tissue",
  title = "PCA based on highly variable cell types",
  n_hv_feat_show = nrow(metadata(se)$clr_hvc)
)
```

Optionally, we can also add an extra dimension and plot the first 3 principal
components:

```{r 3D PCA}
plot_pca3d(se, label_col = "Tissue")
```

## Quantifying group separation

Sample separation (based on a user-defined grouping) can be quantified with
different metrics, for example:

-   Analysis of similarities (ANOSIM)
-   Adjusted rand index (ARI)
-   Modularity score
-   Average silhouette score (not recommended as it expects gaussian cluster
    shapes and is not robust to outliers. ANOSIM and modularity are more
    robust.)

```{r Quantifying group separation, message=FALSE, warning=FALSE}
# You can get the separation scores like this:
dist_mat <- dist(assay(se, "clr"))
# Or use the pre-calculated distance matrix based on the CLR abundances:
dist_mat <- metadata(se)$sample_distances

labels <- colData(se)$Tissue

anosim_r_score <- calc_anosim(dist_mat, labels)
ari_score <- calc_ari(dist_mat, labels)
# modularity_score <- calc_modularity(dist_mat, labels)
silhouette_score <- calc_sil(dist_mat, labels)

print(paste("ANOSIM R:", anosim_r_score))
print(paste("ARI:", ari_score))
# print(paste("Modularity:", modularity_score))
print(paste("Silhouette:", silhouette_score))
```

## Box and bar plots

Box plots are another useful tool to visualize the distribution of cell types
across samples:

```{r Box plot}
plot_boxplot(se, title = "CLR Abundance by Cell Type")
```

If there are biological groupings of interest, we can directly perform
statistical tests between the per-sample cell type compositions:

```{r Grouped box plot}
plot_boxplot(
  se,
  label_col = "Tissue",
  title = "CLR Abundance by Cell Type (with Wilcoxon Test)"
)
```

This can also be plotted as stacked bar plots but it is a bit harder to read.

```{r Bar plot by group}
# Plotting average cell type abundance by experimental group
plot_barplot(
  se,
  label_col = "Tissue",
  plot_by = "group",
  title = "Mean Relative Abundance by Condition"
)
```

```{r Bar plot by sample}
# Plotting cell type abundance for each sample separately
plot_barplot(
  se,
  label_col = "Tissue",
  plot_by = "sample",
  title = "Relative Abundance for Each Sample"
)
```

**NOTE:** It is highly recommended to only do statistical testing on
log-ratio-transformed compositional data and not on counts or relative
abundances in % [@greenacre2021].

The main focus of scECODA is **exploratory** (unsupervised) analysis of samples
based on cell type composition. If you want to do down-stream comparative
(supervised) differential abundance testing of *a priori known groups*
(including statistical designs with multiple known covariates), there are other
tools available for that purpose, e.g. scCODA [@büttner2021] or tascCODA
[@tasccoda].

## Heatmap

The clustered heatmap plot is a good alternative to not only visualize how
samples and cell types correlate but also to cluster samples. It is important
not to re-scale in order to avoid amplifying tiny differences.

Also, it is possible that you see your samples forming separate clusters in the
PCA or heatmap even though you do not find any (or only few) significant
differences in cell type composition. However, there might be small differences
among a large number of cell types which, taken all together, can still drive
sample separation and explain differences between your biological groups.

NOTE: you can specify multiple metadata columns in "label_col" to visualize how
samples group across multiple covariates simultaneously.

Here is an example for a simple dataset:

```{r Plot heatmap example 1}
plot_heatmap(
  se,
  label_col = c("Clinical.efficacy.", "Tissue"),
  cutree_rows = 3,
  cutree_cols = 3
)
```

We see, for example, an enrichment of plasma cells in the tumor samples, while
neutrophils and platelets are more abundant in blood samples.

As a second example, here's the same analysis using a large healthy cohort with
868 PBMC samples and 69 cell types [@gong2024]:

```{r Plot heatmap example 2, fig.height=16, fig.width=12}
se <- ecoda(
  data = example_data$Gongsharma_full$cell_counts$authors_HR,
  metadata = example_data$Gongsharma_full$metadata
)

plot_heatmap(
  se,
  label_col = c("subject.cmv", "age_group"),
  # Additional arguments for pheatmap:
  cutree_rows = 3,
  cutree_cols = 5,
  show_colnames = FALSE
)
```

Subsetting to only the most highly variable cell types (HVCs) yields a very
similar sample clustering, while being much more interpretable.

```{r Plot heatmap example 2 only HVCs}
plot_heatmap(
  se,
  assay = "clr_hvc",
  label_col = c("subject.cmv", "age_group"),
  # Additional arguments for pheatmap:
  cutree_rows = 3,
  cutree_cols = 4,
  show_colnames = FALSE
)
```

Interestingly, we see that cell type composition tends to cluster subjects, in
an unsupervised manner, based on their age group and their CMV status. For
example, naive cell types appear to be enriched in younger subjects that were
not exposed to CMV infection.

## Cell type correlation

The cell type correlation plot offers the possibility to visualize which cell
types correlate with each other, highlighting possible microenvironment hubs
that might be of relevance to explain possible groups of samples in your data:

```{r Correlation plot, fig.height=16, fig.width=16}
plot_corr(se)
```

While the correlation plot below showing only the HVCs is easier to read, it is
nevertheless interesting to see the full correlation plot above. It seems that
there is additional information that might be overlooked:

Besides the main cell type clusters of effector memory cells (separating samples
by cytomegalovirus (CMV) infection status) and SOX4+ naive T cells (separating
samples by age), there seems to be a cluster of ISG+ cell types, memory T cells
and monocytes/DCs whose abundance strongly correlate, respectively.

```{r Correlation plot only HVCs, fig.height=16, fig.width=16}
plot_corr(se, assay = "clr_hvc")
```

## Highly variable cell types

As demonstrated in the heatmap of the extensive Gong & Sharma dataset, not all
cell subtypes contribute equally to sample separation.

Cell types at the top exhibit **high variance** across samples, indicating they
are the main drivers of group separation, while those toward the bottom show
minor variation.

This principle is directly analogous to the selection of highly variable genes
(HVGs) in (sc)RNA-seq data: **high variance is a powerful proxy for biological
relevance.**

We expect cell types that are differentially abundant between key biological
groups to display this increased variance. scECODA provides convenient functions
to calculate and visualize these cell type variances and mean abundance across
all samples:

```{r Mean-variance plot 50 percent}
library(ggplot2)
plot_varmean(se, labels = "none") +
  ggtitle("Default - cell types explaining 50% variance")
```

```{r Mean-variance plot 80 percent}
se_new <- find_hvcs(se, variance_explained = 0.8)
plot_varmean(se_new) +
  ggtitle("Cell types explaining 80% variance")
```

```{r Mean-variance plot top 5 HVCs}
se_new <- find_hvcs(se, top_n_hvcs = 5)
plot_varmean(se_new) +
  ggtitle("Top 5 HVCs")
```

# scECODA and pseudobulk

**ECODA is much more interpretable** (10-100 cell types) compared to gene
expression (2'000 - 20'000 genes). However, pseudobulk analysis has the
advantage of not requiring cell type annotations.

The scECODA package provides convenient functions to explore sample structure at
the population level based on cell type composition but also by default
automatically calculates DESeq2-normalized [@love2015] sample pseudobulk gene
expression.

A PCA plot based on pseudobulk can be plotted like this:

```{r Load dataset including pseudobulk calculation, message=FALSE, warning=FALSE}
# Loading an example SingleCellExperiment dataset
library(scRNAseq)

all.ds <- as.data.frame(surveyDatasets())
ds <- 1
sce_object <- fetchDataset(
  all.ds[["name"]][ds],
  all.ds[["version"]][ds]
)

se <- ecoda(
  data = sce_object,
  sample_col = "sample",
  celltype_col = "cluster",
  get_pb = TRUE
)
```

```{r PCA plot showing top 5 features}
plot_pca(
  se,
  label_col = "Condition",
  title = "PCA based on cell type composition",
  n_hv_feat_show = 5
)
```

```{r PCA plot based on pseudobulk}
plot_pca(
  se,
  assay = "pb",
  label_col = "Condition",
  title = "PCA based on pseudobulk gene expression"
  #, n_hv_feat_show = 5 # optionally, show top n most highly variable genes
)
```

# FAQ

## Can ECODA be affected by batch effect?

Yes, ECODA can be influenced by technical variation between batches. A "batch
effect" can manifests as an artificial inflation or depletion of specific cell
types due to sample handling rather than biology.

Common examples include:

-   Technical artifacts: Red blood cells or platelets resulting from tissue
    contamination or varied bleeding during surgery/dissection.
-   High-variance/Fragile types: Some cell types, e.g. neutrophils, can be
    sensitive to dissociation protocols, resulting in inconsistent recovery
    rates across samples.

Removing these "nuisance" populations can help mitigating noise and/or batch
effect, and to better capture true biological variation.

## Should I remove red blood cells, platelets and neutrophils?

Yes, we recommended to exclude cell types that introduce technical noise rather
than biological signal.

## How can I remove specific cell types?

The recommended workflow depends on your data source and whether you require
pseudobulk analysis:

-   From a Count Matrix: Simply remove the unwanted columns (cell types) from
    your matrix before passing it to ecoda().
-   From Seurat/SCE (with Pseudobulk): If you set get_pb = TRUE, you must subset
    your Seurat or SCE object first. This ensures that the gene expression
    aggregation (pseudobulk) only includes the cells you are interested in.
-   From an existing ECODA object: You can easily extract the data, filter it,
    and re-initialize a clean object:

```{r Removing cell types, eval=FALSE}
# Load example datasets
data(example_data)
Stephenson <- example_data$Stephenson
counts <- Stephenson$cell_counts$authors_LR
metadata <- Stephenson$metadata
se <- ecoda(data = counts, metadata = metadata)

# Specify cell types to drop
cts_to_drop <- c("RBC", "Platelets", "Neutrophils")
keep <- !rownames(assay(se, "counts")) %in% cts_to_drop
counts_subset <- assay(se, "counts")[, keep]
colData_subset <- as.data.frame(colData(se))[keep, ]

# Re-initialize a new subset object
# This ensures all transformations (CLR, HVCs, etc.) are recalculated correctly
se_subset <- ecoda(data = counts_subset, metadata = colData_subset)
```

# References

# Session Info

```{r sessionInfo}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{ecoda_helper}
\alias{ecoda_helper}
\title{Core constructor for SummarizedExperiment objects from count/frequency
matrices}
\usage{
ecoda_helper(
  data = NULL,
  data_is_freq,
  variance_explained = 0.5,
  top_n_hvcs = NULL,
  pseudo_count = 0.5,
  pseudo_frac_min = 2/3,
  add_to = c("all", "zeros")
)
}
\arguments{
\item{data}{A matrix or data frame where \strong{columns are samples} and
\strong{rows are cell types}.}

\item{data_is_freq}{Logical. If \code{TRUE}, \code{data} is treated as
proportions (0-1 or 0-100). If \code{FALSE}, treated as raw integer counts.}

\item{variance_explained}{Numeric (default: 0.5). Target variance for HVCs.}

\item{top_n_hvcs}{Integer (optional). Number of top HVCs to select.}

\item{pseudo_count}{Numeric (default: 0.5). Used if \code{rep_method =
"counts"}.}

\item{pseudo_frac_min}{Numeric (default: 2/3). The multiplier for the minimum
non-zero value when \code{rep_method = "frac_min"}.}

\item{add_to}{Character string. Should the value be added to \code{"all"}
cells or just the \code{"zeros"}?}
}
\value{
A fully initialized
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object.
}
\description{
This is the internal engine that initializes an
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object. It performs zero-imputation, Centered Log-Ratio (CLR) transformation,
calculates sample distances, and identifies highly variable cell types
(HVCs).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{plot_boxplot}
\alias{plot_boxplot}
\title{Generates Boxplots for CLR-transformed Cell Type Abundances with Optional
Group Comparison.}
\usage{
plot_boxplot(
  se,
  assay = c("clr", "clr_hvc", "asin_sqrt"),
  label_col = NULL,
  selected_celltypes = NULL,
  title = "",
  stat_method = "wilcox.test",
  paired = FALSE,
  signif_label = c("p.signif", "p.format")
)
}
\arguments{
\item{se}{An
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object containing the CLR-transformed abundances in the \code{clr}.}

\item{assay}{Character string (default: \code{"clr"}). The name of the assay
in the \code{SummarizedExperiment} object to use for plotting. Must be one
of: \code{"clr"} (CLR-transformed abundances, default),  \code{"clr_hvc"}
(CLR-transformed abundances of only the most highly variable cell types
(HVCs)) or \code{"asin_sqrt"} (arcsin-square root transformed data).}

\item{label_col}{Character string (optional, default: \code{NULL}). The name
of a column in \code{colData(se)} used to define groups for comparison. If
\code{NULL}, a single boxplot is generated per cell type.}

\item{selected_celltypes}{Specify selected celltypes you want to plot instead
of plotting boxplots for all. Note: this does not recalculate CLR-values on
subset.}

\item{title}{Character string (default: \code{""}). The main title for the
plot.}

\item{stat_method}{Character string (default: \code{"wilcox.test"}). The
statistical method used for comparisons between 2 groups (e.g., "t.test",
"wilcox.test"). Note: This is overridden by "kruskal.test" for 3+ groups.}

\item{paired}{Logical (default: \code{FALSE}). If \code{TRUE}, performs a
paired statistical test (e.g., paired t-test or paired Wilcoxon test). Only
applicable for 2-group comparisons.}

\item{signif_label}{Character string (default: \code{"p.signif"}). Controls
how p-values are displayed for 2-group comparisons (e.g., "p.signif" for
stars, "p.format" for numeric p-value).}
}
\value{
A \code{ggplot} object (enhanced by \code{ggpubr}) representing the
CLR abundance boxplot, including jittered data points and dynamic
significance information (pairwise stars for 2 groups, overall p-value for
3+ groups).
}
\description{
This function visualizes the distribution of CLR-transformed abundance for
each cell type using boxplots, optionally splitting the data by a sample
metadata column and performing statistical tests for comparison between
groups.
}
\details{
\strong{Statistical Test Logic:}
\itemize{
\item If the number of groups (\code{label_col} levels) is \strong{2}, the
function uses the specified \code{stat_method}
(default: "wilcox.test") and displays pairwise significance labels
or stars.
\item If the number of groups is \strong{3 or more}, and
\code{stat_method} is "wilcox.test" or "t.test", the function
automatically switches to the global non-parametric test,
\strong{"kruskal.test"}, and displays the overall p-value
for the comparison across all groups.
}
}
\examples{
data(example_data)
se <- ecoda(
    data = example_data$Zhang$cell_counts$authors_LR,
    metadata = example_data$Zhang$metadata,
)

# 1. Boxplots for CLR abundance without grouping (no stats calculated):
plot_boxplot(se)

# 2. Boxplots grouped by 'Treatment' (2 groups) and applying Wilcoxon test:
plot_boxplot(
    se,
    label_col = "Tissue",
    stat_method = "wilcox.test",
    title = "CLR Abundance by Tissue (with Wilcoxon Test)"
)
}
\seealso{
\code{\link{create_long_data}},
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{get_sample_metadata}
\alias{get_sample_metadata}
\title{Extracts constant metadata for each sample from a cell-level data frame.}
\usage{
get_sample_metadata(cell_data_df, sample_col)
}
\arguments{
\item{cell_data_df}{A data frame containing cell-level metadata. This should
include the sample ID column and all potential metadata columns.}

\item{sample_col}{A character string specifying the name of the column that
defines the unique sample ID for each cell (e.g., "Sample_ID").}
}
\value{
A new data frame where:
\itemize{
\item Each row corresponds to a unique sample from the input data.
\item The row names are set to the values of the input
\code{sample_col}.
\item Columns contain only the metadata fields that were constant
across all cells within \strong{each} sample. The \code{sample_col}
itself is excluded from the final columns but used for
row names.
}
}
\description{
This function identifies columns in a cell-level metadata data frame that
have a \strong{constant} value for all cells belonging to the same sample. It
aggregates this constant information, returning a new data frame where each
row represents a unique sample. Columns that vary within any sample are
excluded from the output.
}
\examples{
# Assuming you have a data frame 'cell_df'
cell_df <- data.frame(
    Cell_ID = paste0("C", seq(10)),
    Sample_ID = c(rep("S1", 5), rep("S2", 5)),
    Age = c(rep(30, 5), rep(45, 5)),
    Gender = c(rep("M", 5), rep("F", 5)),
    Cell_Type = c(rep("A", 3), rep("B", 2), rep("A", 3), rep("B", 2))
)

# The 'Age' and 'Gender' columns are constant within each sample (S1 and S2).
# The 'Cell_ID' and 'Cell_Type' columns vary within sample S1 and/or S2.

sample_meta <- get_sample_metadata(cell_df, "Sample_ID")
print(sample_meta)
# Output will have 'Age' and 'Gender' as columns,
# with row names 'S1' and 'S2'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{create_long_data}
\alias{create_long_data}
\title{Reshapes ECODA data into a long format for plotting and analysis.}
\usage{
create_long_data(
  se,
  assay = c("clr", "freq", "asin_sqrt", "clr_hvc"),
  label_col = NULL
)
}
\arguments{
\item{se}{An initialized
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object.}

\item{assay}{Character string (default: \code{"clr"}). The name of the assay
in the \code{SummarizedExperiment} object to use for plotting. Must be one
of: \code{"clr"} (CLR-transformed abundances, default), \code{"clr_hvc"}
(CLR-transformed abundances of only the most highly variable cell types
(HVCs)), \code{"counts"} (raw counts), \code{"counts_imp"} (imputed
counts), \code{"freq"} (relative frequencies), \code{"freq_imp"} (imputed
frequencies), or \code{"asin_sqrt"} (arcsin-square root transformed data).}

\item{label_col}{Character string (optional, default: \code{NULL}). The name
of a column in \code{colData(se)} to merge into the long data frame (e.g.,
"Disease_State" or "Batch"). If \code{NULL}, only the abundance data and
sample/celltype IDs are returned.}
}
\value{
A tidy, long format data frame with columns:
\itemize{
\item \code{sample_id}: Sample identifier.
\item \code{celltype}: Cell type name.
\item \code{rel_abundance} or \code{clr_abundance}:
The quantitative value depending on the chosen
\code{data_slot}.
\item \code{...}: Additional column specified by
\code{label_col} (if provided).
}
}
\description{
This function takes either the relative abundance (\code{freq}) or
CLR-transformed abundance (\code{clr}) matrix from an
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object, converts it from a wide (cell types x samples) to a long (sample,
cell type, value) format for plotting, and optionally joins it with a
specified column from the sample metadata.
}
\seealso{
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{calculate_pseudobulk}
\alias{calculate_pseudobulk}
\title{Calculate Pseudobulk from Count Matrix}
\usage{
calculate_pseudobulk(count_matrix, sample_ids, min_cells = 10)
}
\arguments{
\item{count_matrix}{A gene x cell count matrix (can be a dense matrix or
sparse Matrix). Gene identifiers should be row names and cell barcodes
should be column names.}

\item{sample_ids}{A vector of sample identifiers, one for each column (cell)
in \code{count_matrix}. The length must equal \code{ncol(count_matrix)}.
Must not contain \code{NA} values.}

\item{min_cells}{Minimum number of cells required per sample (default: 1).
Samples with fewer cells than this threshold will be excluded from the
final pseudobulk matrix.}
}
\value{
A gene x sample pseudobulk count matrix. The columns correspond to
the unique sample IDs, and the rows correspond to the genes.
}
\description{
This function aggregates single-cell count data into a "pseudobulk" matrix by
summing the counts for all cells belonging to the same sample ID. It is
robust to both dense and sparse count matrices. It also includes filtering
logic to exclude samples that do not meet a minimum cell count threshold.
}
\examples{
nrow <- 100
ncol <- 100
vals <- nrow * ncol
mat <- round(matrix(exp(rpois(vals, lambda = 3)), nrow = nrow, ncol = ncol))
rownames(mat) <- paste0("Gene", seq_len(ncol))
colnames(mat) <- paste0("Cell", seq_len(nrow))
ids <- rep(c("Sample1", "Sample2"), each = nrow / 2)
pb <- calculate_pseudobulk(count_matrix = mat, sample_ids = ids)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.r
\docType{data}
\name{example_data}
\alias{example_data}
\title{Example Data for scECODA}
\format{
A named \code{list} where each element corresponds to a different
study (e.g., \code{$Adams}). Each study element is a \code{list} containing
the following:
\itemize{
\item \code{cell_counts_highresolution}: Data frame of cell type counts at
high resolution annotation (samples as rows, cell types as columns).
\item \code{cell_counts_lowresolution}: Data frame of cell type counts at
low resolution annotation (samples as rows, cell types as columns).
\item \code{metadata}: Data frame of sample-level metadata (samples as
rows).
\item \code{main_biologicalcondition_columnname}: Character string,
indicating the main biological condition column name in the metadata.
}
}
\source{
Data originally derived from multiple public single-cell datasets and
re-processed as described in the accompanying scECODA manuscript (Halter et
al., \emph{in preparation}).
}
\usage{
data(example_data)
}
\description{
A collection of single-cell compositional data from multiple studies,
structured to demonstrate the full scECODA workflow.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{plot_pca}
\alias{plot_pca}
\title{Plot Principal Component Analysis and Calculate Clustering Scores}
\usage{
plot_pca(
  se,
  assay = c("clr", "counts", "counts_imp", "freq", "freq_imp", "asin_sqrt", "clr_hvc",
    "pb"),
  label_col = NULL,
  scale. = FALSE,
  title = NULL,
  title_show_n_features = TRUE,
  legend_title = "Group",
  show_label_samples = FALSE,
  score_digits = 3,
  cluster_score = FALSE,
  mod_score = FALSE,
  sil_score = FALSE,
  anosim_score = TRUE,
  anosim_permutations = 99,
  anosim_parallel = 1,
  ari_nclusts = NULL,
  knn_k = 3,
  pointsize = 3,
  labelsize = 4,
  coord_equal = TRUE,
  axes = c(1, 2),
  invisible = c("var", "quali"),
  geom = "point",
  n_hv_feat_show = Inf,
  repel = TRUE
)
}
\arguments{
\item{se}{A
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object.}

\item{assay}{Character string (default: \code{"clr"}). The name of the assay
in the \code{SummarizedExperiment} object to use for PCA. Must be one of:
\code{"clr"} (CLR-transformed abundances, default), \code{"clr_hvc"}
(CLR-transformed abundances of only the most highly variable cell types
(HVCs)), \code{"pb"} (pseudobulk gene expression), \code{"counts"} (raw
counts), \code{"counts_imp"} (imputed counts), \code{"freq"} (relative
frequencies), \code{"freq_imp"} (imputed frequencies), or
\code{"asin_sqrt"} (arcsin-square root transformed data).}

\item{label_col}{Character string (optional, default: \code{NULL}). The name
of a column in \code{colData(sd)} used to color and group samples in the
plot, and for calculating clustering scores.}

\item{scale.}{Logical (default: \code{FALSE}). A value indicating whether the
variables should be scaled to have unit variance before the PCA.}

\item{title}{Character string (optional, default: \code{NULL}). The main
title for the plot. If clustering scores are calculated, they are appended
to this title.}

\item{title_show_n_features}{Logical (optional, default: \code{TRUE}) Show
the number of features (cell types or genes) used.}

\item{legend_title}{Character string (default: \code{"Group"}). The title for
the color legend in the plot when a grouping column (\code{label_col}) is
provided.}

\item{show_label_samples}{Logical (default: \code{FALSE}). If \code{TRUE},
sample names are displayed next to the points in the plot. This
automatically adds \code{"text"} to the \code{geom} parameter if it is not
already present.}

\item{score_digits}{Integer (default: \code{3}). The number of decimal places
to round the clustering and ANOSIM scores appended to the plot title.}

\item{cluster_score}{Logical (default: \code{TRUE}). If \code{TRUE},
calculates the Adjusted Rand Index (ARI) using \code{\link{calc_ari}}.}

\item{mod_score}{Logical (default: \code{TRUE}). If \code{TRUE}, calculates
the adjusted Modularity score using \code{\link{calc_modularity}}.}

\item{sil_score}{Logical (default: \code{FALSE}). If \code{TRUE}, calculates
the average Silhouette width using \code{\link{calc_sil}}.}

\item{anosim_score}{Logical (default: \code{TRUE}). If \code{TRUE},
calculates the ANOSIM statistic (R) using \code{vegan::anosim}.}

\item{anosim_permutations}{Integer (default: \code{99}). The number of
permutations to use when calculating the ANOSIM statistic.}

\item{anosim_parallel}{Integer (default: \code{1}). The number of parallel
processes/cores to use for the ANOSIM calculation.}

\item{ari_nclusts}{Integer (optional, default: \code{NULL}). The target
number of clusters (\code{k}) to use for \code{hclust} and \code{pam}. If
\code{NULL}, it defaults to the number of unique levels in \code{labels}.}

\item{knn_k}{Integer (optional, default: \code{3}). The number of nearest
neighbors (\code{k}) to use for the Shared Nearest Neighbor (SNN) graph
construction, required for Modularity score calculation. If \code{NULL}, it
defaults to \code{max(3, round(sqrt(N)))}, where \code{N} is the number of
samples.}

\item{pointsize}{Numeric (default: \code{3}). Size of the points in the plot.}

\item{labelsize}{Numeric (default: \code{4}). Size of the variable labels in
the plot.}

\item{coord_equal}{Logical (default: \code{TRUE}). If \code{TRUE}, forces the
aspect ratio of the plot to be equal.}

\item{axes}{Numeric vector (default: \code{c(1, 2)}). The principal
components to plot (e.g., \code{c(1, 2)} for PC1 vs PC2).}

\item{invisible}{Character vector (default: \code{c("var", "quali")}).
Elements to hide. Can include "var" (variables/cell types), "ind"
(samples), or "quali" (group centroids). see the description of
\link[factoextra:fviz_pca]{factoextra} for details.}

\item{geom}{Character string or vector (default: \code{"point"}). The
geometry to be used for the plot. Allowed values are combinations of:
\itemize{
\item \code{"point"} to show points for individuals (samples)
\item \code{"text"} to show labels for individuals (samples)
\item \code{"arrow"} to show vectors for variables (features)
}
The default \code{"point"} plots points for both individuals and variables.
Use \code{c("point", "text")} to show both points and labels for samples.}

\item{n_hv_feat_show}{Integer (default: \code{Inf}). Number of most highly
variable features to show based on their contribution to the selected axes.}

\item{repel}{Logical (default: \code{TRUE}). Whether to use \code{ggrepel} to
prevent label overlap for variable names.}
}
\value{
A \code{ggplot} object via \code{factoextra} visualizing the PCA
results.
}
\description{
Performs Principal Component Analysis (PCA) on a selected data
matrix from the \code{SummarizedExperiment} object (default:
CLR-transformed abundances, \code{clr}) and visualizes the results. It can
also calculate and display several metrics to evaluate the separation of
groups defined by \code{label_col}. It uses
\link[factoextra:fviz_pca]{factoextra}.
}
\details{
The clustering metrics (ARI, Modularity, Silhouette, ANOSIM) assess
how well the sample groupings (\code{labels}) align with the underlying
data structure in the feature space defined by the PCA.
}
\examples{
data(example_data)
se <- ecoda(
    data = example_data$Zhang$cell_counts$authors_LR,
    metadata = example_data$Zhang$metadata,
)

plot_pca(
    se,
    label_col = "Tissue",
    title = "PCA based on cell type composition",
    anosim_parallel = 1,
    n_hv_feat_show = 5 # Shows the most highly variable features (cell types)
)

# Using only the most highly variable cell types
plot_pca(
    se,
    assay = "clr_hvc",
    label_col = "Tissue",
    title = "PCA based on highly variable cell types",
    anosim_parallel = 1,
    n_hv_feat_show = nrow(S4Vectors::metadata(se)$clr_hvc)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{deseq2_normalize}
\alias{deseq2_normalize}
\title{DESeq2 Normalization of Pseudobulk Data}
\usage{
deseq2_normalize(
  pb,
  metadata = NULL,
  deseq2_design = NULL,
  hvg = NULL,
  nvar_genes = 2000
)
}
\arguments{
\item{pb}{A gene x sample pseudobulk count matrix (Genes as rows, Samples as
columns). Must contain non-negative integer counts.}

\item{metadata}{A data.frame with sample-level metadata. Row names must
exactly match the column names of \code{pb}. The function will reorder the
metadata to match \code{pb}.}

\item{deseq2_design}{Optional: A \code{formula} object specifying the design
used by \code{DESeq2} to estimate size factors and variance for the VST.
This formula should reference columns in the \code{metadata} data frame. If
\code{NULL} (the default), the minimal design \code{~ 1} is used. Note: the
design will be considered for pseudobulk normalization as the function uses
vst(blind = FALSE, ...) internally.}

\item{hvg}{Optional character vector of gene names to use as highly variable
genes. If provided, only these genes will be returned after VST.}

\item{nvar_genes}{Number of top variable genes to select (default: 2000).
Only used if \code{hvg = NULL}; the function will select the top
\code{nvar_genes} by variance after VST.}
}
\value{
A normalized expression matrix (VST-transformed) with\
\strong{samples as columns} and \strong{genes as rows}.
}
\description{
This function normalizes a gene x sample pseudobulk count matrix using the
Variance Stabilizing Transformation (VST) from the \code{DESeq2} package. It
estimates size factors and variance for all genes, performs the VST, and then
subsets the results to include only highly variable genes (HVCs), either
specified by the user or automatically selected based on variance.
}
\details{
\strong{Note:} If \code{deseq2_design} is not specified, the VST is performed using a
minimal design formula (\code{~ 1}) for size factor and variance estimation.
}
\examples{
nrow <- 100
ncol <- 100
vals <- nrow * ncol
mat <- round(matrix(exp(rpois(vals, lambda = 3)), nrow = nrow, ncol = ncol))
rownames(mat) <- paste0("Gene", seq_len(ncol))
colnames(mat) <- paste0("Cell", seq_len(nrow))
ids <- rep(c("Sample1", "Sample2"), each = nrow / 2)
pb <- calculate_pseudobulk(count_matrix = mat, sample_ids = ids)
pb_norm <- deseq2_normalize(pb)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{plot_varmean}
\alias{plot_varmean}
\title{Generates a Mean-Variance Plot for CLR-transformed Cell Type Data.}
\usage{
plot_varmean(
  se,
  plot_title = "",
  highlight_hvcs = TRUE,
  labels = c("only_hvc", "all", "none"),
  plot_fit_line = FALSE,
  smooth_method = "lm"
)
}
\arguments{
\item{se}{A
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object containing pre-calculated cell type variances in
\code{metadata(se)$celltype_variances} and the HVC list in
\code{metadata(se)$hvcs}}

\item{plot_title}{Character string (default: ""). The title for the plot.}

\item{highlight_hvcs}{Logical (default: \code{TRUE}). If \code{TRUE}, the
points corresponding to the Highly Variable Cell Types (HVCs) stored in the
SummarizedExperiment object are colored red.}

\item{labels}{Character (default: "only_hvc"). Options: "all" (label every
point), "none" (no labels), or "only_hvc" (label only the highly variable
cell types).}

\item{plot_fit_line}{Logical (default: \code{FALSE}). If \code{TRUE}, a
smoothing regression line is added to the plot.}

\item{smooth_method}{Character string (default: "lm"). The smoothing method
to use for the regression line if \code{plot_fit_line} is \code{TRUE}
(e.g., "lm" for linear model, "loess" for local regression).}
}
\value{
A \code{ggplot} object representing the mean-variance plot.
}
\description{
This function visualizes the relationship between the mean
abundance (CLR) and the variance (CLR) for each cell type, which is
typically used to identify Highly Variable Cell Types (HVCs).
}
\details{
If \code{highlight_hvcs} is \code{TRUE}, cell types previously
identified and stored in \code{metadata(se)$hvcs} will be highlighted in
red on the plot.
}
\examples{
data(example_data)
se <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)

# 1. Generate the plot, highlighting HVCs (default)
plot_varmean(se, plot_title = "HVCs on Mean-Variance Plot")

# 2. Generate the plot without highlighting HVCs and add a fit line
plot_varmean(se,
    highlight_hvcs = FALSE,
    plot_fit_line = TRUE,
    smooth_method = "loess"
)
}
\seealso{
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment},
\code{\link{get_celltype_variances}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{calc_ari}
\alias{calc_ari}
\title{Calculate Adjusted Rand Index (ARI)}
\usage{
calc_ari(dist_mat, labels, nclusts = NULL, digits = 3, return_mean = TRUE)
}
\arguments{
\item{dist_mat}{Sample distance matrix.}

\item{labels}{Vector of factors or character strings. The true or known
cluster assignments for each row in the matrix.}

\item{nclusts}{Integer (optional, default: \code{NULL}). The target number of
clusters (\code{k}) to use for \code{hclust} and \code{pam}. If
\code{NULL}, it defaults to the number of unique levels in \code{labels}.}

\item{digits}{Integer (default: \code{3}). The number of decimal places to
round the result.}

\item{return_mean}{Logical (default: \code{TRUE}). If \code{TRUE}, returns
the mean of the ARI scores from \code{hclust} and \code{pam}. If
\code{FALSE}, returns a named list with both individual scores.}
}
\value{
A numeric value (mean ARI) or a list of two ARI scores. ARI ranges
from
-1 (disagreement) to +1 (perfect agreement).
}
\description{
Calculates the Adjusted Rand Index (ARI) to measure the agreement between the
known cluster assignments (\code{labels}) and the cluster assignments derived
from two unsupervised clustering methods: Hierarchical Clustering
(\code{hclust}) and Partitioning Around Medoids (\code{pam}).
}
\examples{
library(SummarizedExperiment)
data(example_data)
se <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)

# Extract necessary components
dist_mat <- dist(t(assay(se, "clr")))
labels <- colData(se)$subject.cmv

# Run the calculation
calc_ari(dist_mat, labels)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{ecoda}
\alias{ecoda}
\title{Create an SummarizedExperiment object from various data types}
\usage{
ecoda(
  data = NULL,
  data_is_freq = FALSE,
  metadata = NULL,
  sample_col = NULL,
  celltype_col = NULL,
  get_pb = FALSE,
  variance_explained = 0.5,
  top_n_hvcs = NULL,
  pseudo_count = 0.5,
  pseudo_frac_min = 2/3,
  add_to = NULL
)
}
\arguments{
\item{data}{The primary input, which can be:
\itemize{
\item A \code{Seurat} object.
\item A \code{SingleCellExperiment} object.
\item A cell type x sample matrix or data frame (counts or frequencies).
}}

\item{data_is_freq}{Logical (default: \code{FALSE}). If \code{TRUE}, the
input \code{data} is treated as relative frequencies (proportions) rather
than raw counts.}

\item{metadata}{A data frame containing sample-level metadata. Required if
\code{data} is a matrix/data frame. If \code{data} is a single-cell object,
metadata is extracted automatically.}

\item{sample_col}{The metadata column name defining unique sample IDs.
Required if \code{data} is a single-cell object.}

\item{celltype_col}{The metadata column name defining cell type annotations.
Required if \code{data} is a single-cell object.}

\item{get_pb}{Logical (default: \code{FALSE}). If \code{TRUE}, calculates and
stores DESeq2-normalized pseudobulk data in \code{metadata(se)$pb}}

\item{variance_explained}{Numeric (default: 0.5). Proportion of variance used
to determine the number of highly variable cell types (HVCs).}

\item{top_n_hvcs}{Integer (optional). If provided, specifies the exact number
of top HVCs to select, overriding \code{variance_explained}.}

\item{pseudo_count}{Numeric (default: 0.5). Used if \code{rep_method =
"counts"}.}

\item{pseudo_frac_min}{Numeric (default: 2/3). The multiplier for the minimum
non-zero value when \code{rep_method = "frac_min"}.}

\item{add_to}{Character string. Should the value be added to \code{"all"}
cells or just the \code{"zeros"}?}
}
\value{
A new
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object.
}
\description{
This is a smart constructor function used to initialize an
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object. It handles the processing of single-cell objects (\code{Seurat} or
\code{SingleCellExperiment}) or raw data frames to extract cell type counts,
calculate sample metadata, and optionally generate DESeq2-normalized
pseudobulk data.
}
\examples{
data(example_data)
Zhang <- example_data$Zhang
counts <- Zhang$cell_counts$authors_LR
freq <- calc_freq(counts)
metadata <- Zhang$metadata

se <- ecoda(data = counts, metadata = metadata)

se <- ecoda(data = freq, metadata = metadata)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{calc_clr}
\alias{calc_clr}
\title{Perform the Centered Log-Ratio (CLR) transformation.}
\usage{
calc_clr(df)
}
\arguments{
\item{df}{A data frame or matrix of strictly positive relative frequencies or
counts (samples/observations as columns, components as rows).}
}
\value{
A data frame of the same dimensions containing the CLR-transformed
values.
}
\description{
The CLR transformation is a common technique used for compositional data
analysis, mapping the data from the simplex to Euclidean space. It is defined
as the log of the ratio between each component and the geometric mean of all
components in that sample. \strong{Note:} This function assumes the input data is
strictly positive (i.e., does not contain zeros). Use an imputation or
pseudocount method prior to this function if zeros are present.
}
\examples{
freq_imp <- data.frame(A = c(10.1, 89.9), B = c(50.1, 49.9))
calc_clr(freq_imp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{get_hvcs}
\alias{get_hvcs}
\title{Selects Highly Variable Cell Types (HVCs) based on variance or count
threshold.}
\usage{
get_hvcs(df_var, variance_explained = 0.5, top_n_hvcs = NULL)
}
\arguments{
\item{df_var}{A data frame (typically the output of
\code{\link{get_celltype_variances}}) that must contain at least the
columns \code{celltype} and \code{variance_exp} (cumulative variance
explained) and must be sorted by variance in descending order.}

\item{variance_explained}{Numeric (default: 0.5). The cumulative proportion
of total variance that the selected HVCs must explain. This parameter is
ignored if \code{top_n_hvcs} is set.}

\item{top_n_hvcs}{Integer or Numeric (optional).
\itemize{
\item If an integer (>= 1), it specifies the exact
number of top cell types to select.
\item If a numeric value between 0 and 1, it specifies
the proportion of cell types to select
(e.g., 0.1 for the top 10\%).
}}
}
\value{
A character vector containing the names of the selected highly
variable cell types. The function ensures that at least two cell types are
always returned.
}
\description{
This is a utility function that selects the most variable cell types from a
variance-ranked data frame. Selection can be controlled either by defining
the cumulative variance to be explained or by specifying a fixed number of
cell types.
}
\examples{
data(example_data)
ecoda_object <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)

# Calculate variances
df_var <- get_celltype_variances(ecoda_object)

# Select cell types explaining at least 60\% of variance:
hvcs_60 <- get_hvcs(df_var, variance_explained = 0.6)

# Select the top 10 cell types:
hvcs_top10 <- get_hvcs(df_var, top_n_hvcs = 10)

# Select the top 5\% of cell types:
hvcs_prop <- get_hvcs(df_var, top_n_hvcs = 0.05)
}
\seealso{
\code{\link{find_hvcs}}, \code{\link{get_celltype_variances}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{find_hvcs}
\alias{find_hvcs}
\title{Identifies and stores Highly Variable Cell Types (HVCs) in an
SummarizedExperiment object.}
\usage{
find_hvcs(se, variance_explained = 0.5, top_n_hvcs = NULL)
}
\arguments{
\item{se}{An initialized
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object containing the CLR-transformed data in the \code{clr} assay.}

\item{variance_explained}{Numeric (default: 0.5). The target cumulative
proportion of total variance to be explained by the selected HVCs. The
function stops selecting cell types once this threshold is met.}

\item{top_n_hvcs}{Integer (optional). If provided, this overrides
\code{variance_explained} and selects exactly the top N cell types with the
highest variance.}
}
\value{
The updated
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object with the following metadata populated:
\itemize{
\item \code{celltype_variances}: Data frame of cell type variances.
\item \code{variance_explained}: The exact variance proportion
captured by the selected HVCs.
\item \code{top_n_hvcs}: The number of HVCs selected.
\item \code{hvcs}: Character vector of the names
of the selected HVCs.
}
}
\description{
This function calculates the variance of each cell type across all samples
based on the Centered Log-Ratio (CLR) transformed data. It then selects the
subset of highly variable cell types (HVCs) that collectively explain a
specified proportion of the total variance, or a fixed number of top cell
types.
}
\examples{
data(example_data)
se <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)

# Select HVCs that explain 75\% of the total variance
se <- find_hvcs(se, variance_explained = 0.75)

# Select exactly the top 5 most variable cell types
se <- find_hvcs(se, top_n_hvcs = 5)
}
\seealso{
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment},
\code{\link{get_celltype_variances}}, \code{\link{get_hvcs}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{calc_sil}
\alias{calc_sil}
\title{Calculate Average Silhouette Width}
\usage{
calc_sil(dist_mat, labels, digits = 3)
}
\arguments{
\item{dist_mat}{Sample distance matrix.}

\item{labels}{Vector of factors or character strings. The cluster assignments
for each row in \code{dist_mat} (i.e., the grouping variable).}

\item{digits}{Integer (default: \code{3}). The number of decimal places to
round the final score.}
}
\value{
A numeric value representing the mean silhouette width, typically
ranging from -1 (poor clustering) to +1 (excellent clustering).
}
\description{
Calculates the average silhouette width for a given feature matrix, using
pre-defined cluster assignments (\code{labels}). This metric assesses the
quality of the clustering.
}
\examples{
library(SummarizedExperiment)
data(example_data)
se <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)

# Extract necessary components
dist_mat <- dist(t(assay(se, "clr")))
labels <- colData(se)$subject.cmv

# Run the calculation
calc_sil(dist_mat, labels)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{replace_zeros}
\alias{replace_zeros}
\title{Replace zero values in count or frequency data}
\usage{
replace_zeros(
  df,
  rep_method = c("counts", "frac_min"),
  pseudo_count = 0.5,
  pseudo_frac_min = 2/3,
  add_to = NULL
)
}
\arguments{
\item{df}{A data frame or matrix where zeros need to be replaced.}

\item{rep_method}{Character string specifying the replacement strategy. One
of \code{"counts"}, or \code{"frac_min"}.}

\item{pseudo_count}{Numeric (default: 0.5). Used if \code{rep_method =
"counts"}.}

\item{pseudo_frac_min}{Numeric (default: 2/3). The multiplier for the minimum
non-zero value when \code{rep_method = "frac_min"}.}

\item{add_to}{Character string. Should the value be added to \code{"all"}
cells or just the \code{"zeros"}?}
}
\value{
A data frame or matrix of the same dimensions as \code{df} with all
zero values replaced by the calculated replacement value.
}
\description{
This function replaces zero values in a data frame or matrix to facilitate
downstream transformations that require strictly positive values, such as the
Centered Log-Ratio (CLR) transformation.
}
\details{
The function provides two methods for handling zeros:
\itemize{
\item \strong{counts:} Adds a fixed value (\code{pseudo_count}).
\item \strong{frac_min:} Replaces zeros with a fraction
(\code{pseudo_frac_min}) of the smallest observed non-zero value in
the dataset.
}
}
\examples{
# Replace zeros in a count matrix
counts_df <- data.frame(A = c(10, 0, 5), B = c(20, 10, 0))
replace_zeros(counts_df, rep_method = "counts", add_to = "all")

# Replace zeros in a frequency matrix
freq_df <- data.frame(A = c(0.5, 0.5), B = c(0.2, 0.8), C = c(0.0, 1.0))
replace_zeros(freq_df, rep_method = "frac_min", add_to = "zeros")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{compute_snn_graph}
\alias{compute_snn_graph}
\title{Compute Shared Nearest Neighbor (SNN) Graph}
\usage{
compute_snn_graph(knn)
}
\arguments{
\item{knn}{A matrix of nearest neighbor indices, typically generated by
\code{compute_KNN_from_dist}.}
}
\value{
An \code{igraph} graph object where edge weights correspond to the
count of shared neighbors between nodes.
}
\description{
Constructs a graph where nodes represent samples and edges are weighted
by the number of shared nearest neighbors (SNN) between them.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{plot_corr}
\alias{plot_corr}
\title{Plot Cell Type Correlation Matrix}
\usage{
plot_corr(
  se,
  assay = c("clr", "counts", "counts_imp", "freq", "freq_imp", "asin_sqrt", "clr_hvc",
    "pb"),
  order = "hclust",
  hclust.method = "ward.D2",
  ...
)
}
\arguments{
\item{se}{A
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object.}

\item{assay}{Character string (default: \code{"clr"}). The name of the assay
in the \code{SummarizedExperiment} object to use for the correlation plot
Must be one of: \code{"clr"} (CLR-transformed abundances, default),
\code{"clr_hvc"} (CLR-transformed abundances of only the most highly
variable cell types (HVCs)), \code{"pb"} (pseudobulk gene expression),
\code{"counts"} (raw counts), \code{"counts_imp"} (imputed counts),
\code{"freq"} (relative frequencies), \code{"freq_imp"} (imputed
frequencies), or \code{"asin_sqrt"} (arcsin-square root transformed data).}

\item{order}{Character string (default: \code{"hclust"}). The ordering method
for the correlation matrix. Common options include:
\itemize{
\item \code{"original"} (no reordering)
\item \code{"hclust"} (hierarchical clustering)
\item \code{"FPC"} (first principal component order)
}}

\item{hclust.method}{Character string (default: \code{"ward.D2"}). The
hierarchical clustering method to use if \code{order} is set to
\code{"hclust"}.}

\item{...}{Additional arguments passed to \code{corrplot::corrplot} for plot
customization (e.g., \code{method}, \code{type}, \code{tl.col}).}
}
\value{
A plot object generated by \code{corrplot::corrplot}, which is a base
R plot or a grid object depending on the \code{corrplot} version and options.
}
\description{
Calculates the pairwise Pearson correlation matrix for all cell
types (columns) using the Centered Log-Ratio (CLR) transformed abundances
stored in \code{assay(se, "clr")}. It then visualizes this matrix as a
heatmap using \code{corrplot::corrplot}.
}
\details{
The function uses the CLR matrix, where high correlation between two
cell types suggests they vary together across samples, indicating potential
co-occurrence or co-regulation.
}
\examples{
data(example_data)
# Example of a large cohort with 868 samples and 69 cell types
se <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)
plot_corr(se)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{compute_KNN_from_dist}
\alias{compute_KNN_from_dist}
\title{Compute K-Nearest Neighbors (KNN) from Distance Matrix}
\usage{
compute_KNN_from_dist(dist_mat, knn_k)
}
\arguments{
\item{dist_mat}{A square distance matrix or an object of class \code{dist}.}

\item{knn_k}{Integer. The number of neighbors (\code{k}) to return for each
sample.}
}
\value{
A matrix with \code{nrow(dist_mat)} rows and \code{knn_k} columns,
where each row contains the indices of the nearest neighbors.
}
\description{
Identifies the indices of the K-nearest neighbors for each sample based on a
pre-computed distance matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{calc_anosim}
\alias{calc_anosim}
\title{Analysis of Similarities (ANOSIM) R score}
\usage{
calc_anosim(dist_mat, labels, permutations = 99, parallel = 1, digits = 3)
}
\arguments{
\item{dist_mat}{Sample distance matrix.}

\item{labels}{Vector of factors or character strings. The grouping variable
(the known cluster assignments) for each row in the matrix.}

\item{permutations}{Integer (default: \code{99}). The number of permutations
to use when calculating the ANOSIM R-statistic and p-value.}

\item{parallel}{Integer (optional, default: \code{1}). The number
of parallel processes/cores to use for the permutation testing.}

\item{digits}{Integer (default: \code{3}). The number of decimal places to
round the R-statistic.}
}
\value{
A numeric value representing the \strong{ANOSIM R-statistic}.
}
\description{
Calculates the ANOSIM R-statistic to test whether there is
significant separation between two or more groups (defined by
\code{labels}) based on the sample distance matrix (\code{dist_mat}).
}
\details{
ANOSIM compares the mean of rank dissimilarities between groups to
the mean of rank dissimilarities within groups. The R-statistic ranges from
-1 to 1:
\itemize{
\item An R value close to \strong{1} indicates clear separation of groups.
\item An R value close to \strong{0} indicates that the separation is
no greater than expected by chance (i.e., poor separation).
\item An R value close to \strong{-1} indicates that within-group
dissimilarities are greater than between-group dissimilarities
(a very rare result).
}
}
\examples{
library(SummarizedExperiment)
data(example_data)
se <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)

# Extract necessary components
dist_mat <- dist(t(assay(se, "clr")))
labels <- colData(se)$subject.cmv

# Run the calculation
calc_anosim(dist_mat, labels, parallel = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scECODA-package.R
\docType{package}
\name{scECODA-package}
\alias{scECODA}
\alias{scECODA-package}
\title{scECODA: Single-Cell Exploratory Compositional Data Analysis}
\description{
The scECODA R package provides a complete workflow for the analysis and visualization of compositional data, primarily focusing on cell type proportions derived from single-cell data. It implements specialized methods, such as the Centered Log-Ratio (CLR) transformation, to properly analyze proportional data while avoiding the biases introduced by the compositional constraint. The package encapsulates data management, transformation, and analysis into a single SummarizedExperiment object, offering downstream tools for dimensionality reduction via PCA, calculating critical metrics like the Adjusted Rand Index (ARI) and Modularity to quantify sample grouping quality, and generating high-quality visualizations like heatmaps and scatter plots.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/carmonalab/scECODA}
  \item Report bugs at \url{https://github.com/carmonalab/scECODA/issues}
}

}
\author{
\strong{Maintainer}: Christian Halter \email{scecoda.dev@gmail.com} (\href{https://orcid.org/0009-0009-5479-2246}{ORCID})

Authors:
\itemize{
  \item Massimo Andreatta \email{Massimo.Andreatta@unige.ch} (\href{https://orcid.org/0000-0002-8036-2647}{ORCID})
  \item Santiago Carmona \email{Santiago.Carmona@unige.ch} (\href{https://orcid.org/0000-0002-2495-0671}{ORCID})
}

Other contributors:
\itemize{
  \item Swiss Cancer Research Foundation [funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{plot_heatmap}
\alias{plot_heatmap}
\title{Generates a Heatmap of Cell Abundance Data from an SummarizedExperiment
assay.}
\usage{
plot_heatmap(
  se,
  assay = c("clr", "counts", "counts_imp", "freq", "freq_imp", "asin_sqrt", "clr_hvc",
    "pb"),
  label_col,
  cluster_rows = TRUE,
  cluster_cols = TRUE,
  scale = "none",
  clustering_method = "ward.D2",
  angle_col = "90",
  ...
)
}
\arguments{
\item{se}{A
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object.}

\item{assay}{Character string (default: \code{"clr"}). The name of the assay
in the \code{SummarizedExperiment} object to use for the heatmap. Must be
one of: \code{"clr"} (CLR-transformed abundances, default),
\code{"clr_hvc"} (CLR-transformed abundances of only the most highly
variable cell types (HVCs)), \code{"pb"} (pseudobulk gene expression),
\code{"counts"} (raw counts), \code{"counts_imp"} (imputed counts),
\code{"freq"} (relative frequencies), \code{"freq_imp"} (imputed
frequencies), or \code{"asin_sqrt"} (arcsin-square root transformed data).}

\item{label_col}{Character string. The name of the column in
\code{colData(se)} to use for annotating the samples (columns) of the
heatmap.}

\item{cluster_rows}{Logical (default: \code{TRUE}). Whether to apply
hierarchical clustering to the cell types (rows).}

\item{cluster_cols}{Logical (default: \code{TRUE}). Whether to apply
hierarchical clustering to the samples (columns).}

\item{scale}{Character string (default: \code{"none"}). Method for scaling
the abundance values within the heatmap using \code{pheatmap}. Options
include \code{"none"}, \code{"row"}, or \code{"column"}. Note: The data is
internally \strong{mean-centered} (\code{scale(center=TRUE, scale=FALSE)})
across samples before being passed to \code{pheatmap}, regardless of this
argument.}

\item{clustering_method}{Character string (default: \code{"ward.D2"}). The
clustering method to use for hierarchical clustering. Options are passed
directly to \code{hclust} (e.g., \code{"complete"}, \code{"average"},
\code{"ward.D2"}).}

\item{angle_col}{Character string (default: \code{"90"}). Angle of the sample
labels (columns).}

\item{...}{Additional arguments passed directly to the \code{pheatmap}
function.}
}
\value{
A \code{pheatmap} object, which is typically visualized automatically
when called interactively.
}
\description{
This function visualizes a data matrix from a specified slot (e.g.,
CLR-transformed, frequency, or pseudobulk data) after mean-centering. It
supports optional filtering to only Highly Variable Cell Types (HVCs) and
includes a sample annotation sidebar based on a specified metadata column.
}
\examples{
# Example for a simple dataset:
data(example_data)
se <- ecoda(
    data = example_data$Zhang$cell_counts$authors_LR,
    metadata = example_data$Zhang$metadata,
)

plot_heatmap(se, label_col = c("Clinical.efficacy.", "Tissue"))

plot_heatmap(
    se,
    label_col = c("Clinical.efficacy.", "Tissue"),
    # Additional arguments for pheatmap:
    cutree_rows = 3,
    cutree_cols = 3
)

# Example of a large cohort with 868 samples and 69 cell types
se <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)

plot_heatmap(
    se,
    label_col = c("subject.cmv", "age_group"),
    cutree_rows = 3,
    cutree_cols = 5,
    show_colnames = FALSE
)

# Using only the most highly variable cell types (HVCs)
plot_heatmap(
    se,
    assay = "clr_hvc",
    label_col = c("subject.cmv", "age_group"),
    cutree_rows = 3,
    cutree_cols = 4,
    show_colnames = FALSE
)
}
\seealso{
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment},
\code{\link[pheatmap]{pheatmap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{get_celltype_counts}
\alias{get_celltype_counts}
\title{Get the cell type counts from a long data frame (e.g. seurat object metadata)
where each cell is a row.}
\usage{
get_celltype_counts(cell_data_df, sample_col, celltype_col)
}
\arguments{
\item{cell_data_df}{A data frame where each row represents a single cell, and
columns contain cell-level information, including sample ID and cell type
annotation.}

\item{sample_col}{The column that defines the sample ID for each cell}

\item{celltype_col}{The column that defines the cell type annotation for each
cell}
}
\value{
A data frame with cell types as rows and samples as columns,
containing the count of each cell type per sample.
}
\description{
Get the cell type counts from a long data frame (e.g. seurat object metadata)
where each cell is a row.
}
\examples{
# Create example data frame
cell_data_df <- data.frame(
    Cell_ID = paste0("C", seq(10)),
    Sample_Name = c(rep("S1", 5), rep("S2", 5)),
    Cluster_Annotation = factor(c(
        "B_Cell", "T_Cell", "B_Cell", NA, "T_Cell",
        "T_Cell", "Macrophage", "B_Cell", "T_Cell", "T_Cell"
    ))
)
#
# Calculate cell type counts per sample
celltype_counts <- get_celltype_counts(
    cell_data_df = cell_data_df,
    sample_col = "Sample_Name",
    celltype_col = "Cluster_Annotation"
)
#
print(celltype_counts)
# Note how the NA cell type count is handled and renamed to "NA".
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{get_ecoda_assay}
\alias{get_ecoda_assay}
\title{Helper to get assay for ecoda SummarizedExperiment objects}
\usage{
get_ecoda_assay(se, assay)
}
\arguments{
\item{se}{A
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object.}

\item{assay}{Character string, the name of the assay.}
}
\value{
A data frame containing the assay data.
}
\description{
Because not all data can be stored in assays (e.g. pseudobulk
and clr_hvc have different number of features (rows) than counts etc.)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{plot_barplot}
\alias{plot_barplot}
\title{Generates a Stacked Bar Plot of Cell Type Relative Abundance.}
\usage{
plot_barplot(
  se,
  label_col = NULL,
  plot_by = c("sample", "group"),
  custom_sample_order = NULL,
  title = "",
  facet_by_label_col = TRUE
)
}
\arguments{
\item{se}{A
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object containing cell type relative frequencies in the \code{freq} assay.}

\item{label_col}{Character string (optional, default: \code{NULL}). The name
of a column in \code{colData(se))} used to define grouping or groups
(required if \code{plot_by = "group"}).}

\item{plot_by}{Character string (default: \code{"sample"}). Specifies whether
to plot the relative abundance for each individual sample (\code{"sample"})
or the average relative abundance aggregated by a group (\code{"group"})
defined by \code{label_col}.}

\item{custom_sample_order}{Character vector (optional, default: \code{NULL}).
A vector of sample IDs to enforce a specific order when \code{plot_by =
  "sample"}. If \code{NULL}, samples are ordered first by \code{label_col}
(if provided) and then naturally.}

\item{title}{Character string (default: \code{""}). The main title for the
plot.}

\item{facet_by_label_col}{Logical (default: \code{TRUE}). If \code{TRUE} and
\code{plot_by = "sample"}, the plot will be faceted (split) horizontally by
the categories in \code{label_col}.}
}
\value{
A \code{ggplot} object representing the stacked bar plot.
}
\description{
This function visualizes the relative abundance (composition) of cell types
across samples or across aggregated groups. It automatically handles data
preparation and ordering based on the provided parameters.
}
\examples{
data(example_data)
se <- ecoda(
    data = example_data$Zhang$cell_counts$authors_LR,
    metadata = example_data$Zhang$metadata,
)

plot_barplot(se)

# Plotting average cell type abundance by experimental group
plot_barplot(
    se,
    label_col = "Tissue",
    plot_by = "group",
    title = "Mean Relative Abundance by Condition"
)

# Plotting cell type abundance for each sample separately
plot_barplot(
    se,
    label_col = "Tissue",
    plot_by = "sample",
    title = "Relative Abundance for Each Sample"
)
}
\seealso{
\code{\link{create_long_data}},
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{plot_pca3d}
\alias{plot_pca3d}
\title{Plot 3-dimensional interactive Principal Component Analysis plot}
\usage{
plot_pca3d(
  se,
  assay = c("clr", "counts", "counts_imp", "freq", "freq_imp", "asin_sqrt", "clr_hvc",
    "pb"),
  label_col = NULL,
  scale. = FALSE
)
}
\arguments{
\item{se}{A
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object.}

\item{assay}{Character string (default: \code{"clr"}). The name of the assay
in the \code{SummarizedExperiment} object to use for PCA. Must be one of:
\code{"clr"} (CLR-transformed abundances, default), \code{"clr_hvc"}
(CLR-transformed abundances of only the most highly variable cell types
(HVCs)), \code{"pb"} (pseudobulk gene expression), \code{"counts"} (raw
counts), \code{"counts_imp"} (imputed counts), \code{"freq"} (relative
frequencies), \code{"freq_imp"} (imputed frequencies), or
\code{"asin_sqrt"} (arcsin-square root transformed data).}

\item{label_col}{Character string (optional, default: \code{NULL}). The name
of a column in \code{colData(se)} used to color and group samples in the
plot, and for calculating clustering scores.}

\item{scale.}{Logical (default: \code{FALSE}). A value indicating whether the
variables should be scaled to have unit variance before the PCA.}
}
\value{
An interactive 3D \code{plotly} object visualizing the PCA results.
}
\description{
Performs Principal Component Analysis (PCA) on a selected data
matrix from the \code{ECODA} object (default: CLR-transformed abundances,
\code{clr}) and visualizes the results in 3D colored by \code{label_col}.
}
\examples{
data(example_data)
se <- ecoda(
    data = example_data$Zhang$cell_counts$authors_LR,
    metadata = example_data$Zhang$metadata,
)

plot_pca3d(se, label_col = "Tissue")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{get_celltype_variances}
\alias{get_celltype_variances}
\title{Calculates the variance of cell types across samples.}
\usage{
get_celltype_variances(se, descending = TRUE)
}
\arguments{
\item{se}{An initialized
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object containing CLR-transformed data in the \code{clr} assay}

\item{descending}{Logical (default: \code{TRUE}). If \code{TRUE}, the
returned data frame is sorted by variance in descending order.}
}
\value{
A data frame where each row is a cell type, containing:
\itemize{
\item \code{celltype}: The name of the cell type.
\item \code{avg_clr_abundance}: The mean CLR value for that
cell type across all samples.
\item \code{Variance}: The variance of the CLR values for that
cell type across all samples.
\item \code{cumulative_variance}: Cumulative sum of the variance,
ranked by variance.
\item \code{variance_exp}: The proportion of total variance
explained cumulatively.
}
}
\description{
This function takes the Centered Log-Ratio (CLR) transformed cell type
abundance data from an
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object, calculates the mean CLR abundance and variance for each cell type. It
also calculates the cumulative variance explained by the cell types when
ranked by variance.
}
\examples{
data(example_data)
se <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)

# Calculate variances without plotting, sorted by ascending variance
df_var_asc <- get_celltype_variances(
    se,
    descending = FALSE
)
}
\seealso{
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment},
\code{\link{plot_varmean}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.r
\name{calc_modularity}
\alias{calc_modularity}
\title{Calculate Adjusted Modularity Score}
\usage{
calc_modularity(dist_mat, labels, knn_k = 3, digits = 3)
}
\arguments{
\item{dist_mat}{Sample distance matrix.}

\item{labels}{Vector of factors or character strings. The cluster assignments
for each row in \code{dist_mat}.}

\item{knn_k}{Integer (optional, default: \code{NULL}). The number of nearest
neighbors (\code{k}) used for SNN graph construction. If \code{NULL}, it
defaults to \code{max(3, round(sqrt(N)))}, where \code{N} is the number of
samples.}

\item{digits}{Integer (default: \code{3}). The number of decimal places to
round the final adjusted score.}
}
\value{
A numeric value representing the adjusted modularity score,
ranging from -0.5 to 1.0.
}
\description{
Calculates the Modularity score for a given clustering (\code{labels}) based
on a Shared Nearest Neighbor (SNN) graph.The score is adjusted by the
theoretical maximum modularity for the number of groups to always be between
-0.5 (poor clustering) and +1 (excellent clustering)).
}
\examples{
library(SummarizedExperiment)
data(example_data)
se <- ecoda(
    data = example_data$Gongsharma_full$cell_counts$authors_HR,
    metadata = example_data$Gongsharma_full$metadata
)

# Extract necessary components
dist_mat <- dist(t(assay(se, "clr")))
labels <- colData(se)$subject.cmv

# Run the calculation
\dontrun{
calc_modularity(dist_mat, labels)
}
}
\references{
Brandes, Ulrik, et al.
"On finding graph clusterings with maximum modularity."
European Symposium on Algorithms. Springer, Berlin, Heidelberg, 2007.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.r
\name{calc_freq}
\alias{calc_freq}
\title{Calculate relative frequencies (percentages) column-wise.}
\usage{
calc_freq(df)
}
\arguments{
\item{df}{A data frame or matrix of counts (samples/observations as columns,
components as rows). Must contain only numeric, non-negative values.}
}
\value{
A data frame of the same dimensions, where each column sums to 100.
}
\description{
This function takes a matrix or data frame of counts and transforms each
column (assumed to be a sample or observation) into relative frequencies,
expressing each component as a percentage of the column's total sum.
}
\examples{
counts <- data.frame(A = c(10, 90), B = c(50, 50))
calc_freq(counts)
}
